@@ -1,13 +1,31 @@
-# RCS: @(#) $Id: stm.tcl,v 1.6 2003/08/28 13:58:10 barras Exp $
+# RCS: @(#) $Id: stm.tcl,v 1.19 2005/01/04 14:23:04 galliano Exp $
 
 # Copyright (C) 1999-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
 
 ################################################################
-# updated 8 jan. 2001; 27 jun. 2001; 10 jul. 2001
+# conversion rules: updated 8 jan. 2001; 27 jun. 2001; 10 jul. 2001; 26 oct. 2004 ; 09 dc. 2004
+#
+# Export to stm (with option):
+# ----------------------------
+#
+# While using stm export on command line, options can be specified by
+# setting the global variable "stmopt":
+# 
+#   ex: trans -set stmopt value -export stm file 
+#
+#   value can be:
+#
+#       "pron" => Replace the string to which a pronouce event applies to, only
+#                 if the "desc" attributes contains the string: "(...:)".
+#                 ex: www.tf1.fr+[pron="(URL:) WWW point TF1 point fr]
+#                  => WWW point TF1 point FR
+#           
+#       "svl" => in case of overlapping speech, keeps the name of tje 2 speakers
+#                for the SVL task. The segment is always ignorated for transcription task. 
+#
+#       this values are exclusives => it is not possible to have svl and pron at the same time
 #
-# Export to stm:
-# -------------
 #   file format: id channel speaker startTime endTime <o,cond,gender> transcript
 # with
 #   id = stm basename (up to 1st dot) restricted to alphanum chars (else "_")
@@ -69,33 +87,58 @@
 #
 # gaps not indicated by "inter_segment_gap" and shorter than 0.2 sec
 #   are ignored and concatenated to the previous segment
-
+# (this can be disabled using command line option '-set importMinGap 0.0')
 namespace eval stm {
 
-  variable msg "STM format"
-  variable ext ".stm"
-
-  # if set to 1, also process deprecated tag format <lang=...> </lang> etc.
-  variable deprecated 0
-
-  proc export {name} {
+    variable msg "STM format"
+    variable ext ".stm"
+    
+    # if set to 1, also process deprecated tag format <lang=...> </lang> etc.
+    variable deprecated 0
+    
+    proc export {name} {
     global v
-
-    variable head ""
-    variable bgTime ""
-    variable bgLvl "off"
-    variable bgTyp ""
-    variable oldtime ""
-    variable time ""
-    variable cond
-    variable turncond
-    variable txt
-    variable channel [open $name w]
-    variable base
-    variable nontrans ""
-    set nxt ""
-
-    puts $channel \
+	
+	# export any 'event'?
+	setdef v(exportEvent) false
+	
+	# other export option
+	setdef v(stmopt) ""
+	
+	variable head ""
+	variable bgTime ""
+	variable bgLvl "off"
+	variable bgTyp ""
+	variable oldtime ""
+	variable time ""
+	variable cond
+	variable turncond
+	variable txt
+	variable channel [open $name w]
+	variable base
+	variable nontrans ""
+	variable nontime 0
+	variable alnum
+	set nxt ""
+	
+    # provide explicit informations on export and used encoding in header
+	if {![catch {encoding system}]} {
+	    fconfigure $channel -encoding [EncodingFromName $::v(encoding)]
+	    set msg " with encoding $::v(encoding)"
+	} else {
+	    set msg ""
+	}
+	set rev [lindex [split {$Revision: 1.19 $}] 1]
+	puts $channel ";; Transcriber export by stm.tcl,v $rev on [clock format [clock seconds]]$msg"
+	set episode [$v(trans,root) getChilds "element" "Episode"]
+	if {[$episode getAttr program] != "" || [$episode getAttr air_date] != ""} {
+	    puts $channel ";; program [$episode getAttr program] of [$episode getAttr air_date]"
+	}
+	puts $channel ";; transcribed by [$v(trans,root) getAttr scribe], version [$v(trans,root) getAttr version] of [$v(trans,root) getAttr version_date]"
+	puts $channel ";;"
+	
+	# provide header for NIST sclite analysis
+	puts $channel \
 {;; CATEGORY "0" "" ""
 ;; LABEL "O" "Overall" "Overall"
 ;;
@@ -111,177 +154,224 @@ namespace eval stm {
 ;; LABEL "female" "Female" ""
 ;; LABEL "male"   "Male" ""
 ;; LABEL "unknown"   "Unknown" ""}
-
-    # get basename from filename up to first dot,
-    # then keep only alphanumeric chars
-    #set base [$v(trans,root) getAttr "audio_filename"]    
-    set base [lindex [split [file tail $name] .] 0]
-    if {[info tclversion] >= 8.3} {
-      set alnum {[^[:alnum:]]+}
-    } else {
-      set alnum "\[^\x30-\x39\x41-\x5A\x61-\x7A\xC0-\xD6\xD8-\xF6\xF8-\xFF]+"
-    }
-    regsub -all $alnum $base "_" base
-    set base [string trim $base "_"]
-
-    set episode [$v(trans,root) getChilds "element" "Episode"]
-    foreach sec [$episode getChilds "element" "Section"] {
-      # ignore "nontrans" sections
-      if {[$sec getAttr "type"] == "nontrans"} {
-	set t0 [format %.3f [$sec getAttr "startTime"]]
-	set t1 [format %.3f [$sec getAttr "endTime"]]
-	dump $t0
-	ignore $t0 $t1
-	continue
-      }
-      foreach tur [$sec getChilds "element" "Turn"] {
-	set turncond "f0"
-	if {[$tur getAttr "mode"] == "spontaneous"} {
-	  set turncond "f1"
-	}
-	if {[$tur getAttr "channel"] == "telephone"} {
-	  set turncond "f2"
-	}
-	if {[$tur getAttr "fidelity"] == "low"} {
-	  set turncond "f4"
-	}
-	set spk [$tur getAttr "speaker"]
-	set gender ""
-	set scope "global"
-	if {$spk == ""} {
-	  set spk "inter_segment_gap"
-	  set gender ""
-	} elseif {[llength $spk] == 1} {
-	  catch {
-	    set atts [::speaker::get_atts $spk]
-	    set gender [lindex $atts 2]
-	    if {[lsearch -exact {"male" "female"} $gender] < 0} {
-	      set gender "unknown"
-	    }
-	    set scope [lindex $atts 5]
-	    if {[lindex $atts 3] == "nonnative"} {
-	      if {$turncond == "f4"} {
-		set turncond "fx"
-	      } else {
-		set turncond "f5"
-	      }
-	    }
-	  }
-	  # keep only alphanumeric chars in speaker name, replace other by _
-	  set spk [::speaker::name $spk]
-	  regsub -all $alnum $spk "_" spk
-	  set spk [string trim $spk "_"]
-	  # prefix local names by file id
-	  if {$scope != "global" && ![string match ${base}* $spk]} {
-	    set spk "${base}_$spk"
-	  }
+	
+	# get basename from filename up to first dot,
+	# then keep only alphanumeric chars
+	#set base [$v(trans,root) getAttr "audio_filename"]    
+	set base [lindex [split [file tail $name] .] 0]
+	if {[info tclversion] >= 8.3} {
+	set alnum {[^[:alnum:]]+}
 	} else {
-	  # exclude overlapping speech from stm
-	  set t0 [format %.3f [$tur getAttr "startTime"]]
-	  set t1 [format %.3f [$tur getAttr "endTime"]]
-	  dump $t0
-	  ignore $t0 $t1
-	  continue
+	    set alnum "\[^\x30-\x39\x41-\x5A\x61-\x7A\xC0-\xD6\xD8-\xF6\xF8-\xFF]+"
 	}
-	foreach chn [$tur getChilds] {
-	  if {[$chn class] == "data"} {
-	    set data [$chn getData]
-	    regsub -all "\n" $data " " data
-	    if {$nxt != ""} {
-	      if {[regexp { *([^ ]+)( .*)} $data all wrd data]} {
-		append txt [format $nxt $wrd]
-	      }
-	      set nxt ""
+	regsub -all $alnum $base "_" base
+	set base [string trim $base "_"]
+	
+	set episode [$v(trans,root) getChilds "element" "Episode"]
+	foreach sec [$episode getChilds "element" "Section"] {
+	    # ignore "nontrans" sections
+	    if {[$sec getAttr "type"] == "nontrans"} {
+		set t0 [format %.3f [$sec getAttr "startTime"]]
+		set t1 [format %.3f [$sec getAttr "endTime"]]
+		dump $t0
+		ignore $t0 $t1
+		continue
 	    }
-	    if {$txt != "" &&
-		[string index $txt [expr [string length $txt]-1]] != " "} {
-	      append txt " "
-	    }
-	    append txt $data
-	  } elseif {[$chn class] == "element"} {
-	    switch [$chn getType] {
-	      "Background" {
-		set bgTyp [$chn getAttr "type"]
-		set bgLvl [$chn getAttr "level"]		      
-		# detect first bg change after beginning of segment
-		set newtime [format %.3f [$chn getAttr "time"]]
-		if {$newtime == $time} {
-		  setcond
-		} elseif {$newtime > $time && $bgTime == ""} {
-		  set bgTime $newtime
+	    foreach tur [$sec getChilds "element" "Turn"] {
+		set turncond "f0"
+		if {[$tur getAttr "mode"] == "spontaneous"} {
+		    set turncond "f1"
 		}
-	      }
-	      "Sync" {
-		set newtime [format %.3f [$chn getAttr "time"]]
-		if {$time == "" || $newtime > $time} {
-		  set time $newtime
+		if {[$tur getAttr "channel"] == "telephone"} {
+		    set turncond "f2"
 		}
-		dump $time
-		set head "$base 1 $spk $time %s <o,%s,$gender> %s"
-		set oldtime "$time"
-		set bgTime ""
-		setcond
-	      }
-	      "Who" {
-		#set nb [$chn getAttr "nb"]
-		#append txt " \[$nb] "
-	      }
-	      "Comment" {
-		#set desc [$chn getAttr "desc"]
-		#append txt "<comment>$desc</comment>"
-	      }
-	      "Event" {
-		set desc [$chn getAttr "desc"]
-		set type [$chn getAttr "type"]
-		set extn [$chn getAttr "extent"]
-		# replace spaces with _ in description
-		regsub -all "\[ \t\n]+" $desc "_" desc
-		if {$type == "language"} {
-		  catch {set desc $::iso639($desc)}
-		  set f(begin) " \[$type=$desc-] "
-		  set f(end) " \[-$type] "
-		  set f(instantaneous) " \[$type=$desc] "
+		if {[$tur getAttr "fidelity"] == "low"} {
+		    set turncond "f4"
+		}
+		set spk [$tur getAttr "speaker"]
+		set gender ""
+		set scope "global"
+		if {$spk == ""} {
+		    set spk "inter_segment_gap"
+		    set gender ""
+		} elseif {[llength $spk] == 1} {
+		    catch {
+			set atts [::speaker::get_atts $spk]
+			set gender [lindex $atts 2]
+			if {[lsearch -exact {"male" "female"} $gender] < 0} {
+			    set gender "unknown"
+			}
+			set scope [lindex $atts 5]
+			if {[lindex $atts 3] == "nonnative"} {
+			    if {$turncond == "f4"} {
+				set turncond "fx"
+			    } else {
+				set turncond "f5"
+			    }
+			}
+		    }
+
+		    set spk [normalize_name $spk]
 		} else {
-		  set f(begin) " \[$desc-] "
-		  set f(end) " \[-$desc] "
-		  set f(instantaneous) " \[$desc] "
+		    # exclude overlapping speech from stm
+		    set t0 [format %.3f [$tur getAttr "startTime"]]
+		    set t1 [format %.3f [$tur getAttr "endTime"]]
+		    dump $t0
+		    #  exclude overlapping speech from stm but keep the speaker names for SVL task
+		    if {$v(stmopt) == "svl"} {
+			set spk1 [normalize_name [lindex $spk 0]]
+			set spk2 [normalize_name [lindex $spk 1]]
+			ignoreSVL $t0 $t1 $spk1 $spk2
+		    } else {
+			ignore $t0 $t1
+		    }
+		    continue
 		}
-		switch $extn {
-		  "previous" {
-		    if {$type == "noise"} {
-		      if {[regexp {(.* )([^ ]+) *} $txt all txt prv]} {
-			append txt "$f(begin) $prv $f(end)"
-		      }
+		foreach chn [$tur getChilds] {
+		    if {[$chn class] == "data"} {
+			set data [$chn getData]
+			regsub -all "\n" $data " " data
+			if {$nxt != ""} {
+			    if {[regexp { *([^ ]+)( .*)} $data all wrd data]} {
+				append txt [format $nxt $wrd]
+			    }
+			    set nxt ""
+			}
+			if {$txt != "" && [string index $txt [expr [string length $txt]-1]] != " "} {
+			    append txt " "
+			}
+			append txt $data
+		    } elseif {[$chn class] == "element"} {
+			# Only if the gobal variable stmopt is set to "pron" on command line.
+			# If the next event element type is "pronouce" and its extent "previous"
+			# Replace the last word by the "desc" of the element tag, only if this
+			# desc contains (...:) or (cent...), for example (URL:) or (19 cent...).
+			# Note that because of tne NE, some previous event are no more closed to the
+			# word it apllies to, that why there is the test ($|\s$)
+			if {$v(stmopt) == "pron"} {
+			    if {[$chn getType] == "Event" && [$chn getAttr "type"] == "pronounce"} {
+				if {[$chn getAttr "extent"] == "previous"} { 
+				    set desctmp [$chn getAttr "desc"]
+				    # Replace the "19 cent..."
+				    if {[regexp {cent\.\.\.} $desctmp all]} {
+					regexp {([^ ]+)$} $txt all lastwrd
+					if {[regexp {([0-9][0-9])([0-9][0-9])} $lastwrd all cent ten]} {
+					    if { $ten != "00"} {
+						regsub {[^ ]+($|\s$)} $txt "$cent cent $ten" txt
+					    } else {
+						regsub {[^ ]+($|\$)} $txt "$cent cents" txt	
+					    }
+					}
+				    }
+				    # Replace the Roman numbers "VI" or "URL" by the right pron
+				    if {[regexp {^\(.*\:\)(\s*)(.*$)} $desctmp all type subst]} {
+					regexp {([^ ]+)($|\s$)} $txt all lastwrd
+					regsub {([^ ]+)($|\s$)} $txt "$subst" txt
+				    }
+				}
+			    }
+			}
+			switch [$chn getType] {
+			    "Background" {
+				set bgTyp [$chn getAttr "type"]
+				set bgLvl [$chn getAttr "level"]		      
+				# detect first bg change after beginning of segment
+				set newtime [format %.3f [$chn getAttr "time"]]
+				if {$newtime == $time} {
+				    setcond
+				} elseif {$newtime > $time && $bgTime == ""} {
+				    set bgTime $newtime
+				}
+			    }
+			    "Sync" {
+				set newtime [format %.3f [$chn getAttr "time"]]
+				if {$time == "" || $newtime > $time} {
+				    set time $newtime
+				}
+				dump $time
+				set head "$base 1 $spk $time %s <o,%s,$gender> %s"
+				set oldtime "$time"
+				set bgTime ""
+				setcond
+			    }
+			    "Who" {
+				#set nb [$chn getAttr "nb"]
+				#append txt " \[$nb] "
+			    }
+			    "Comment" {
+				#set desc [$chn getAttr "desc"]
+				#append txt "<comment>$desc</comment>"
+			    }
+			    "Event" {
+				set desc [$chn getAttr "desc"]
+				set type [$chn getAttr "type"]
+				set extn [$chn getAttr "extent"]
+				# replace spaces with _ in description
+				regsub -all "\[ \t\n]+" $desc "_" desc
+				if {$type == "language"} {
+				    catch {set desc $::iso639($desc)}
+				    set f(begin) " \[$type=$desc-] "
+				    set f(end) " \[-$type] "
+				    set f(instantaneous) " \[$type=$desc] "
+				} else {
+				    set f(begin) " \[$desc-] "
+				    set f(end) " \[-$desc] "
+				    set f(instantaneous) " \[$desc] "
+				}
+				switch $extn {
+				    "previous" {
+					if {$type == "noise" || $v(exportEvent)} {
+					    if {[regexp {(.* )([^ ]+) *} $txt all txt prv]} {
+						append txt "$f(begin) $prv $f(end)"
+					    }
+					}
+				    }
+				    "next" {
+					if {$type == "noise" || $v(exportEvent)} {
+					    set nxt "$f(begin) %s $f(end)"
+					}
+				    }
+				    "begin" - "end" - "instantaneous" {
+					if {$type == "noise" || $type == "language"|| $v(exportEvent)} {
+					    append txt $f($extn)
+					} elseif {$type == "pronounce" && $extn == "instantaneous"
+						  && ($desc == "pi" || $desc == "pif")} {
+					    append txt $f($extn)
+					}
+				    }
+				}
+			    }
+			}
 		    }
-		  }
-		  "next" {
-		    if {$type == "noise"} {
-		      set nxt "$f(begin) %s $f(end)"
-		    }
-		  }
-		  "begin" - "end" - "instantaneous" {
-		    if {$type == "noise" || $type == "language"} {
-		      append txt $f($extn)
-		    } elseif {$type == "pronounce" && $extn == "instantaneous"
-			      && ($desc == "pi" || $desc == "pif")} {
-		      append txt $f($extn)
-		    }
-		  }
 		}
-	      }
 	    }
-	  }
 	}
-      }
+	if {[info exists tur]} {
+	    dump [format %.3f [$tur getAttr "endTime"]]
+	}
+	if {$nontrans != ""} {
+	    puts stderr "WARNING - unclosed nontrans/language/comment segment starting at $nontime in $name"
+	}
+	close $channel
     }
-    if {[info exists tur]} {
-      dump [format %.3f [$tur getAttr "endTime"]]
-    }
-    if {$nontrans != ""} {
-      puts stderr "Pending non-transcribed segment in $name"
-    }
-    close $channel
-  }
+    
+proc normalize_name {id} {
+    variable base
+    variable alnum
+    # keep only alphanumeric chars in speaker name, replace other by _    
+    set spk  [::speaker::name $id]
+    set atts [::speaker::get_atts $id]
+    set scope [lindex $atts 5]
+    # the following line remove the comment into speaker name wich begins with ","
+    regsub ",.*" $spk "" spk
+    regsub -all $alnum $spk "_" spk
+    set spk [string trim $spk "_"]
+    # prefix local names by file id
+    if {$scope != "global" && ![string match ${base}* $spk]} {
+	set spk "${base}_$spk"
+    } 
+    return $spk
+}
 
   proc setcond {} {
     variable turncond
@@ -317,6 +407,7 @@ namespace eval stm {
     variable channel
     variable base
     variable nontrans
+    variable nontime
     variable deprecated
 
     if {$head != ""} {
@@ -342,6 +433,7 @@ namespace eval stm {
 	# detect beginning of unclosed nontrans or lang or comment segment
 	if {[regexp -nocase "\\\[(nontrans|lang|comment)\[^\]\]*-\]" $txt all typ]} {
 	  set nontrans $typ
+	  set nontime $oldtime
 	} elseif {[regexp -nocase "\\\[(nontrans|lang|conv)(\[^\]-\])*\]" $txt]} {
 	  # exclude segments with nontrans/lang/conv intantaneous tags
 	  set notthis 1
@@ -353,19 +445,29 @@ namespace eval stm {
       }
       if {$notthis || [regexp "^(\\\[\[^\]\]*\]|<\[^>\]*>| )*$" $txt]} {
 	if {$notthis} {
-	  ignore $oldtime $t1
+	  if {$oldtime < $t1} {
+	    ignore $oldtime $t1
+	  }
 	  # detect end of nontrans or lang or comment segment
 	  if {$nontrans != "" && [regexp "\\\[-$nontrans\[^\]\[\]*\]" $txt]} {
 	    set nontrans ""
 	  }
 	} else {
 	  # segment contains only space or tags (except nontrans/lang./conv tags)
-	  puts $channel "$base 1 inter_segment_gap $oldtime $t1 <o,$cond,>"
+	  if {$oldtime < $t1} {
+	    puts $channel "$base 1 inter_segment_gap $oldtime $t1 <o,$cond,>"
+	  }
 	}
       } else {
 	# normalize spacing for text output
 	regsub -all "  +" $txt " " txt
 	puts $channel [format $head $t1 $cond [string trim $txt]]
+	if {$oldtime >= $t1} {
+	  puts stderr "WARNING - non-positive stm segment $oldtime-$t1 in $base.stm"
+	}
+	if {[string first "inter_segment_gap" $head] >= 0 } {
+	  puts stderr "WARNING - transcription for no-speaker stm segment $oldtime-$t1 in $base.stm"	
+	}
       }
     }
     set head ""
@@ -378,9 +480,20 @@ namespace eval stm {
     puts $channel "$base 1 excluded_region $t0 $t1 <o,,unknown> ignore_time_segment_in_scoring"
   }
 
+  proc ignoreSVL {t0 t1 spk1 spk2} {
+    variable channel
+    variable base
+    puts $channel "$base 1 $spk1,$spk2 $t0 $t1 <o,,unknown> ignore_time_segment_in_scoring"
+  }
+
+
   proc import {name} {
     global v
 
+    # intra-speaker inter-segment gaps which are not explicitly marked
+    # and of lower duration than given will be merged with following segment
+    setdef v(importMinGap) 0.2
+
     set file_id ""
     if {$v(sig,name) != ""} {
       set file_id [file root [file tail $v(sig,name)]]
@@ -410,7 +523,10 @@ namespace eval stm {
 	#if {$file_id !="" && $id != $file_id} continue
 	set cnd [split [string tolower [string trim $cnd " \t<>"]] ","]
 	set cond [lindex $cnd 1]
-	if {$begin-$prev < 0.2} {set begin $prev}
+	if {$begin-$prev < $v(importMinGap)} {set begin $prev}
+	# synchro times starting with "+" are floating
+	if {[string index $begin 0] == "+"} {set begin " [string range $begin 1 end]"}
+	if {[string index $end 0] == "+"} {set end "  [string range $end 1 end]"}
 	if {$text == "ignore_time_segment_in_scoring"} {
 	  set speaker ""
 	  set text "\[nontrans]"
@@ -442,7 +558,10 @@ namespace eval stm {
 	    }
 	    $tur setAttr "speaker" [::speaker::create $name "" $gender $dialect "" $scope]
 	  }
+	} elseif {$begin > $prev} {
+	  set sync [::xml::element "Sync" "time $prev" -in $tur]
 	}
+
 	# turn conditions
 	if {$cond == "f1"} {
 	  $tur setAttr "mode" "spontaneous"
@@ -559,6 +678,10 @@ namespace eval stm {
      return $result
    }
 
+   # only needed for compatibility with version <1.4.6
+   proc readSegmt {content} {return [lindex [lindex [readSegmtSet $content] 0] 0]}
+   if {[info commands ::ColorMap] == ""} {proc ::ColorMap c {return}}
+
   # fold adjacent sorted segments with similar label(s) into a single one
   proc unify {list1 {delta 0.1} {lastfield "end"}} {
     set list2 {}
