@@ -1,4 +1,7 @@
-/* Copyright (C) 1998, DGA - part of the Transcriber program
+/* 
+ * RCS: @(#) $Id: segmt.c,v 1.3 2002/08/26 13:27:38 barras Exp $
+ *
+ * Copyright (C) 1998-2000, DGA - part of the Transcriber program
  * distributed under the GNU General Public License (see COPYING file)
  */
 
@@ -23,6 +26,8 @@
 # define MAX(a,b) ((a)>(b)?(a):(b))
 #endif
 
+/* automatic update of named fonts using private or public API */
+#if TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION <= 3
 /* from tkInt.h for automatic update of named fonts */
 typedef Window (TkClassCreateProc) _ANSI_ARGS_((Tk_Window tkwin,
 	Window parent, ClientData instanceData));
@@ -36,6 +41,10 @@ typedef struct TkClassProcs {
 } TkClassProcs;
 EXTERN void		TkSetClassProcs _ANSI_ARGS_((Tk_Window tkwin,
 			    TkClassProcs *procs, ClientData instanceData));
+#else
+#define TkSetClassProcs Tk_SetClassProcs
+#define TkClassProcs Tk_ClassProcs
+#endif
 
 /* --------------------------------------------------------------------- */
 
@@ -183,7 +192,7 @@ static char *SegmtVarProc(ClientData clientData, Tcl_Interp *interp,
 static char *TimeVarProc(ClientData clientData, Tcl_Interp *interp,
                         char *name1, char *name2, int flags);
 static int ParseSegmentVar (Segmt *a);
-Tk_3DBorder GetBorder(Segmt *a, char *colorName, Tk_3DBorder defbord);
+Tk_3DBorder GetBorder(Segmt *a, char *colorName);
 
 /* --------------------------------------------------------------------- */
 
@@ -509,6 +518,9 @@ static int ParseSegmentVar (Segmt *a)   {
       txt = NULL; nbchar = 0;
       if (nbfield >=3) {
 	 txt = Tcl_GetStringFromObj(oneObj[2], &nbchar);
+#ifdef TCL_ENCODING_START
+	 nbchar = Tcl_NumUtfChars(txt, nbchar);
+# endif
       }
       a->listSeg[i].txt = txt;
       a->listSeg[i].len = nbchar;
@@ -518,7 +530,7 @@ static int ParseSegmentVar (Segmt *a)   {
       if (nbfield >= 4) {
 	 char *colorName = Tcl_GetStringFromObj(oneObj[3], NULL);
 	 if ((colorName != NULL) && (strlen(colorName)>0))
-	    border = GetBorder(a, colorName, border);
+	    border = GetBorder(a, colorName);
       }
       a->listSeg[i].col = border;
    }
@@ -665,8 +677,8 @@ TimeVarProc(clientData, interp, name1, name2, flags)
 
 /* --------------------------------------------------------------------- */
 
-/* Hash table for borders */
-Tk_3DBorder GetBorder(Segmt *a, char *colorName, Tk_3DBorder defbord) {
+/* Hash table for borders - NULL value for bogus colors */
+Tk_3DBorder GetBorder(Segmt *a, char *colorName) {
    int new;
    Tk_3DBorder border;
    Tk_Uid uid = Tk_GetUid(colorName);
@@ -674,7 +686,6 @@ Tk_3DBorder GetBorder(Segmt *a, char *colorName, Tk_3DBorder defbord) {
    if (new) {
       /* fprintf(stderr,"Creating border %s\n", colorName); */
       border = Tk_Get3DBorder(a->interp, a->tkwin, uid);
-      if (border==NULL) border=defbord;
       Tcl_SetHashValue(entry, border);      
    } else {
       border = (Tk_3DBorder) Tcl_GetHashValue(entry);      
@@ -689,8 +700,8 @@ static void BorderTableFree(Segmt *a) {
    entry = Tcl_FirstHashEntry(&a->borderTable, &search);
    while (entry != NULL) {
       border = (Tk_3DBorder) Tcl_GetHashValue(entry);
-      /*fprintf(stderr,"Deleting border %s\n", Tk_NameOf3DBorder(border));*/
-      Tk_Free3DBorder(border);
+      /* fprintf(stderr,"Deleting border %s\n", Tk_NameOf3DBorder(border)); */
+      if (border != NULL) Tk_Free3DBorder(border);
       entry = Tcl_NextHashEntry(&search);
    }
    Tcl_DeleteHashTable(&a->borderTable);
@@ -755,13 +766,23 @@ static void SegmtReallyDraw(Segmt *a)
 	    : width * bp[j] / a->length - X0;
       }
 
-      /* Look for newlines to slice text in two parts */
-      txt[1] = strchr(txt[0], '\n');
+      /* Look for form-feeds to slice text in two parts */
+      if (txt[0] != NULL) {
+	txt[1] = strchr(txt[0], '\f');
+      } else {
+	txt[1] = NULL;
+      }
       /* Detect if we need to write on half-height parts */
       if (txt[1] != NULL) {
 	 nbchar[0] = txt[1] - txt[0];
+#ifdef TCL_ENCODING_START
+	 nbchar[0] = Tcl_NumUtfChars(txt[0], nbchar[0]);
+# endif
 	 txt[1] ++;
 	 nbchar[1] = strlen(txt[1]);
+#ifdef TCL_ENCODING_START
+	 nbchar[1] = Tcl_NumUtfChars(txt[1], nbchar[1]);
+# endif
 
 	 y2 = a->pixheight/2;
 	 height = a->height/2;
@@ -776,13 +797,13 @@ static void SegmtReallyDraw(Segmt *a)
 	 gc = a->gc;
       }
 
-      for (j=0; j<2 && txt[j]; j++) {
+      for (j=0; j<2 && (txt[1] || j<1); j++) {
 	 y1 = (j==1 && txt[1] != NULL) ? a->pixheight/2 : 0;
 	 /* Draw polygon */
 	 if (x[3]-x[2] >= 2*bd) {
 	    Tk_Fill3DRectangle( a->tkwin, a->pixmap, border, x[2], y1,
 				x[3]-x[2], y2, bd, a->relief);
-	 } else {
+	 } else if (x[2]>= 0) {
 	    /* Some ad-hoc test to give hashed aspect at very low resolution */
 	    Tk_Fill3DRectangle( a->tkwin, a->pixmap, border, x[2], y1,
 				2*bd, y2, x[2]%2, a->relief);
