@@ -1,4 +1,7 @@
-/* Copyright (C) 1998, DGA - part of the Transcriber program
+/* 
+ * RCS: @(#) $Id: wavfm.c,v 1.3 2002/07/10 14:24:16 barras Exp $
+ *
+ * Copyright (C) 1998-2000, DGA - part of the Transcriber program
  * distributed under the GNU General Public License (see COPYING file)
  */
 
@@ -12,13 +15,16 @@
 #include <tcl.h>
 #include <tk.h>
 
+extern int useOldObjAPI;
+extern int littleEndian;
+
 #define BE_OK(x) {int res = (x); if (res != TCL_OK) return res;}
 
 /* --------------------------------------------------------------------- */
 typedef struct {
-   short min; 
    short max;
-} Minmax;
+   short min;
+} Maxmin;
 
 /* Widget data structure */
 typedef struct {
@@ -46,20 +52,23 @@ typedef struct {
    double cursor;
    double volume;
    char *signal;
+   char *shapename;
 
    /* Graphic contexts, internals */
    int flags;
    int channels;
    int rate;
    Pixmap pixmap;
+   Pixmap pixmap2;
    int pixwidth;
    int pixheight;
    GC gc;
    GC cursorgc;
    GC selectgc;
+   Tcl_Obj *sampObj;
    short  *samples;
    int nbsamp;
-   Minmax *shape;
+   Maxmin *shape;
    XPoint *points;
    int nbshap;
    double end;
@@ -72,15 +81,18 @@ typedef struct {
 #define REDRAW 0x1
 #define REALLY 0x2
 #define FOCUS  0x4
+#define CURSOR 0x8
 
 /* Widget attributes descriptions */
 typedef enum {
-  OPTION_SIGNAME
+  OPTION_SIGNAME, OPTION_SHAPNAME
 } ConfigSpec;
 
 static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_STRING, "-sound", "sound", "sound",
         "", Tk_Offset(Wavfm, signal), 0},
+    {TK_CONFIG_STRING, "-shape", "shape", "shape",
+        "", Tk_Offset(Wavfm, shapename), 0},
 
     {TK_CONFIG_BORDER, "-background", "background", "Background",
         "light blue", Tk_Offset(Wavfm, background),
@@ -157,6 +169,7 @@ static int WavfmInstanceCmd(ClientData clientData, Tcl_Interp *interp,
 static int WavfmConfigure( Tcl_Interp *interp, Wavfm *w,
 		      int argc, char *argv[], int flags);
 static void WavfmDisplay(ClientData clientData);
+static void WavfmAddCursor(Wavfm *w);
 static void WavfmEventProc(ClientData clientData, XEvent *eventPtr);
 static void WavfmDestroy(char *blockPtr);
 
@@ -195,6 +208,7 @@ int WavfmCmd( ClientData clientData, Tcl_Interp *interp,
    wavfmPtr->borderwidth = 0;
    wavfmPtr->font = NULL;
    wavfmPtr->pixmap = 0;
+   wavfmPtr->pixmap2 = 0;
    wavfmPtr->pixwidth = 0;
    wavfmPtr->pixheight = 0;
    wavfmPtr->winwidth = 0;
@@ -210,6 +224,7 @@ int WavfmCmd( ClientData clientData, Tcl_Interp *interp,
    wavfmPtr->selectbegin = 0;
    wavfmPtr->selectend = 0;
    wavfmPtr->signal = NULL;
+   wavfmPtr->shapename = NULL;
    wavfmPtr->padX = 0;
    wavfmPtr->padY = 0;
    wavfmPtr->channels = 1;
@@ -217,6 +232,7 @@ int WavfmCmd( ClientData clientData, Tcl_Interp *interp,
    wavfmPtr->prev_begin = 0;
    wavfmPtr->prev_length = 0;
    wavfmPtr->prev_width = 0;
+   wavfmPtr->sampObj = NULL;
    wavfmPtr->samples = NULL;
    wavfmPtr->nbsamp = 0;
    wavfmPtr->shape = NULL;
@@ -279,6 +295,22 @@ int WavfmInstanceCmd(ClientData clientData, Tcl_Interp *interp,
 	 return WavfmConfigure( interp, w, argc-2, argv+2,
 				 TK_CONFIG_ARGV_ONLY);
       }
+   } else if ((strncmp(argv[1],"cursor",len)==0) && (len>=2)) {
+      if (argc==3) {
+          if (Tcl_GetDouble(interp, argv[2], &(w->cursor)) != TCL_OK)
+            return TCL_ERROR;
+	  if ((w->tkwin != NULL) && Tk_IsMapped( w->tkwin)
+	      && !(w->flags & REDRAW)) {
+	    Tk_DoWhenIdle( WavfmDisplay, (ClientData) w);
+	    w->flags |= REDRAW;
+	  }
+        w->flags |= CURSOR;
+	  return TCL_OK;
+      } else {
+	 Tcl_AppendResult(interp, "wrong # args: should be '", argv[0],
+			  " cursor value'", (char *)NULL);
+	 return TCL_ERROR;
+      }
    } else {
       Tcl_AppendResult(interp, "bad option '", argv[1],
 		       "': must be cget or configure", (char *)NULL);
@@ -295,6 +327,7 @@ static int SendCmd( Wavfm *w, char *cmd)
    Tcl_Obj *cmdObj;
    int result;
 
+   /* fprintf(stderr,"%s\n",cmd); */
    cmdObj = Tcl_NewStringObj( w->signal, -1);
    Tcl_AppendStringsToObj( cmdObj, " ", cmd, NULL);
    Tcl_IncrRefCount( cmdObj);
@@ -312,29 +345,35 @@ static int SendCmd( Wavfm *w, char *cmd)
 static int ReadSignal( Wavfm *w, long pos, long width)
 {
    int size;
-   char cmd[256], *str;
+   char cmd[256], *p = NULL;
+   Tcl_Obj *res;
 
    w->nbsamp = 0;
    if (w->signal == NULL || strlen(w->signal)==0) return -1;
 
-   /*
-     sprintf(cmd, "seek %ld\n", w->channels * pos);
-     sprintf(cmd, "read %ld\n", w->channels * width);
-   */
-   sprintf(cmd, "get %ld %ld\n", pos, width);
+   sprintf(cmd, "datasamples -start %ld -end %ld -byteorder %s\n",
+	   pos, pos+width-1, (littleEndian ? "littleEndian" : "bigEndian"));
    if (SendCmd( w, cmd) != TCL_OK) return -1;
-
-   str = Tcl_GetStringFromObj( Tcl_GetObjResult( w->interp), &size);
+   
+   if (w->sampObj != NULL) {
+      Tcl_DecrRefCount( w->sampObj);
+      w->sampObj = NULL;
+      w->samples = NULL;
+   }
+   res = Tcl_GetObjResult( w->interp);
+   if (useOldObjAPI) {
+      p = Tcl_GetStringFromObj( res, &size);
+   } else {
+#ifdef TCL_81_API
+      p = (char *) Tcl_GetByteArrayFromObj( res, &size);
+#endif
+   }
    if (size == 0) {
       w->nbsamp = 0;
       return 0;
    }
-   if (w->samples != NULL) {
-      w->samples = (short *) Tcl_Realloc( (char *)w->samples, size);
-   } else {
-      w->samples = (short *) Tcl_Alloc( size);
-   }
-   memcpy (w->samples, str, size);
+   Tcl_IncrRefCount(w->sampObj = res);
+   w->samples = (short *) p;
    w->nbsamp = size/(w->channels*sizeof(short));
    return w->nbsamp;
 }
@@ -344,21 +383,21 @@ static int ReadShape( Wavfm *w, int width, double begin, double length)
 {
    int size, c, width2, i, j, base;
    double delta, prev;
-   char cmd[256], *str;
+   char cmd[256], *p = NULL;
 
    w->nbshap = 0;
    if (w->signal == NULL || strlen(w->signal)==0) return -1;
 
    /* Request memory for the right size */
-   size = width * w->channels * sizeof(Minmax);
+   size = width * w->channels * sizeof(Maxmin);
    w->nbshap = width;
    if (size == 0) {
       return 0;
    }
    if (w->shape != NULL) {
-      w->shape = (Minmax *) Tcl_Realloc( (char *)w->shape, size);
+      w->shape = (Maxmin *) Tcl_Realloc( (char *)w->shape, size);
    } else {
-      w->shape = (Minmax *) Tcl_Alloc( size);
+      w->shape = (Maxmin *) Tcl_Alloc( size);
    }
 
    /* Handle scrolling case with minimal load */
@@ -395,23 +434,37 @@ static int ReadShape( Wavfm *w, int width, double begin, double length)
       }
    }
 
-   sprintf(cmd, "shape %d %f %f\n", width, begin, length);
-   if (SendCmd( w, cmd) != TCL_OK) return -1;
+   sprintf(cmd, "shape -width %d -start %ld -end %ld -byteorder %s",
+	   width, (long) floor(w->rate*begin),
+	   (long) ceil(w->rate*(begin+length))-1,
+	   (littleEndian ? "littleEndian" : "bigEndian"));
+   if (w->shapename != NULL && strlen(w->shapename) > 0) {
+      strcat(cmd, " -shape ");
+      strcat(cmd, w->shapename);
+   }
+   strcat(cmd, "\n");
+   if (SendCmd(w, cmd) != TCL_OK) return -1;
 
-   str = Tcl_GetStringFromObj( Tcl_GetObjResult( w->interp), &size);
-   if (size != sizeof(Minmax)*width*w->channels || base+width > w->nbshap) {
-      Tcl_AppendResult( w->interp, "Shape size problem", NULL);
-      Tcl_BackgroundError( w->interp);
+   if (useOldObjAPI) {
+      p = Tcl_GetStringFromObj(Tcl_GetObjResult(w->interp), &size);
+   } else {
+#ifdef TCL_81_API
+      p = (char *) Tcl_GetByteArrayFromObj(Tcl_GetObjResult(w->interp), &size);
+#endif
+   }
+   if (size != sizeof(Maxmin)*width*w->channels || base+width > w->nbshap) {
+      Tcl_AppendResult(w->interp, "Shape size problem", NULL);
+      Tcl_BackgroundError(w->interp);
       w->nbshap = 0;
       /* forget the callback command which doesn't work */
       w->signal[0] = '\0';
       return -1;
    }
-   memcpy (w->shape + base * w->channels, str, size);
+   memcpy (w->shape + base * w->channels, p, size);
    return width;
 }
 
-void AllocPoints( Wavfm *w, int nb)
+static void AllocPoints( Wavfm *w, int nb)
 {
    int size = nb * sizeof(XPoint);
    if (size == 0) return;
@@ -501,8 +554,8 @@ int WavfmConfigure( Tcl_Interp *interp, Wavfm *w,
          && !(w->flags & REDRAW)) {
       Tk_DoWhenIdle( WavfmDisplay, (ClientData) w);
       w->flags |= REDRAW;      
-      w->flags |= REALLY;      
    }
+   w->flags |= REALLY;
    return TCL_OK;
 }
 
@@ -597,17 +650,15 @@ static void WavfmReallyDraw(Wavfm *w)
 	    w->points[i].y = center - w->samples[c + i*w->channels] * vRatio;
 	    /* should do some clipping and change fg color for selection */
 	 }
-	 XDrawLines(w->display, w->pixmap, w->gc, w->points, w->nbsamp, 
-		    CoordModeOrigin);
+	 if (w->nbsamp > 0) {
+	    XDrawLines(w->display, w->pixmap, w->gc, w->points, w->nbsamp, 
+		       CoordModeOrigin);
+	 }
       }
    }
    
-   /* Cursor */
-   if ((w->cursor >= w->begin) && (w->cursor <= w->end)) {
-      x1 = x2 = bdX + width*(w->cursor - w->begin)/w->length;
-      y1 = bd; y2 = w->pixheight-bd-1;
-      XDrawLine(w->display, w->pixmap, w->cursorgc, x1, y1, x2, y2);
-   }
+   /* Draw cursor */
+   WavfmAddCursor(w);
 
    /* Keep previous parameters */
    w->prev_width=width;
@@ -615,6 +666,24 @@ static void WavfmReallyDraw(Wavfm *w)
    w->prev_length=w->length;
 }
 
+/* pixmap2 = pixmap + cursor */
+static void WavfmAddCursor(Wavfm *w)
+{
+   int bd = w->borderwidth;
+   int bdX = bd + w->padX;
+   int width = w->pixwidth-2*bdX-1;
+   int x1, y1, x2, y2;
+
+   XCopyArea( w->display, w->pixmap, w->pixmap2,
+	      w->gc, 0, 0, w->pixwidth, w->pixheight, 0, 0);
+
+   if ((w->cursor >= w->begin) && (w->cursor <= w->end)) {
+     x1 = x2 = bdX + width*(w->cursor - w->begin)/w->length;
+     y1 = bd; y2 = w->pixheight-bd-1;
+     XDrawLine(w->display, w->pixmap2, w->cursorgc, x1, y1, x2, y2);
+   }
+}
+
 /* Display */
 static void WavfmDisplay(ClientData clientData)
 {
@@ -635,11 +704,15 @@ static void WavfmDisplay(ClientData clientData)
       /* fprintf(stderr,"New pixmap %d x %d\n",width,height); */
       w->pixwidth = width;
       w->pixheight = height;
-      /* free old pixmap */
+      /* free old pixmaps */
       if (w->pixmap != 0)
 	 Tk_FreePixmap(w->display, w->pixmap);
       w->pixmap = Tk_GetPixmap(w->display, Tk_WindowId(tkwin),
             width, height, Tk_Depth(tkwin));
+      if (w->pixmap2 != 0)
+	 Tk_FreePixmap(w->display, w->pixmap2);
+      w->pixmap2 = Tk_GetPixmap(w->display, Tk_WindowId(tkwin),
+            width, height, Tk_Depth(tkwin));
    }
    
    if (w->flags & REALLY) {
@@ -648,7 +721,12 @@ static void WavfmDisplay(ClientData clientData)
       w->flags &= ~REALLY;      
    }
 
-   XCopyArea( w->display, w->pixmap, Tk_WindowId(tkwin),
+   if (w->flags & CURSOR) {
+      WavfmAddCursor( w);
+      w->flags &= ~CURSOR;      
+   }
+
+   XCopyArea( w->display, w->pixmap2, Tk_WindowId(tkwin),
 	      w->gc, 0, 0, width, height, 0, 0);
 
    return;
@@ -697,8 +775,8 @@ static void WavfmDestroy(char *blockPtr)
 {
    Wavfm *w = (Wavfm *)blockPtr;
 
-   if (w->samples != NULL) {
-      Tcl_Free( (char *) w->samples);
+   if (w->sampObj != NULL) {
+      Tcl_DecrRefCount( w->sampObj);
    }
    if (w->shape != NULL) {
       Tcl_Free( (char *) w->shape);
@@ -715,6 +793,12 @@ static void WavfmDestroy(char *blockPtr)
    if (w->selectgc != None) {
       Tk_FreeGC(w->display, w->selectgc);
    }
+   if (w->pixmap != 0) {
+     Tk_FreePixmap(w->display, w->pixmap);
+   }
+   if (w->pixmap2 != 0) {
+     Tk_FreePixmap(w->display, w->pixmap2);
+   }
    if (w->flags & REDRAW) {
       Tk_CancelIdleCall( WavfmDisplay, (ClientData) w);
    }
