@@ -2,7 +2,7 @@
 #\
 exec wish "$0" "$@"
 
-# RCS: @(#) $Id: Menu.tcl,v 1.11 2003/08/27 14:55:39 barras Exp $
+# RCS: @(#) $Id: Menu.tcl,v 1.22 2005/01/05 09:51:46 mantam Exp $
 
 # Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
@@ -87,7 +87,13 @@ proc add_menu {m liste} {
                   || [regsub "A(lt)?-" $arg "Alt-" arg]
                   || [regsub "S(hift)?-" $arg "Shift-" arg]} {
                      lappend option -accelerator [string toupper $arg]
-                     set sequence ""
+                  }
+                  foreach {short long} {
+                     "C(trl)?-" "Command-"
+                     "A(lt)?-" "Option-" 
+                     "S(hft)?-" "Shift-"
+                  } {
+                     regsub $short $sequence $long sequence
                   }
                } else {
                   lappend option -accelerator $arg
@@ -204,8 +210,18 @@ proc InitMenus {} {
 	 {"Informations"	cmd {CreateInfoFrame}}
 	 {"Edit episode attributes..."	cmd {EditEpisode}}	
 	 {""}
+	 {"Open video file..." 			cmd {OpenVideoFile}}
 	 {"Open audio file..." 	-bind "Ctrl-a"	cmd {OpenAudioFile}}
-	 {"Save audio selection as..."          cmd {SaveAudioSegment as}}
+	 {"Synchronized audio files"		cascade {
+	   {"Add audio file..." 	cmd {OpenAudioFile add}}
+	   {""}
+	 }}
+	 {"Save audio segment(s)"  cascade {
+		{"Selected..."     cmd   {SaveAudioSegment}}
+		{"Automatic..." cmd   {SaveAudioSegmentAuto}}
+	 }}
+	 {""}
+	 {"Open segmentation file..." 	cmd {OpenSegmt}}
 	 {""}
 	 {"Quit"		-bind "Ctrl-q"	cmd {Quit}}
       }}
@@ -220,23 +236,26 @@ proc InitMenus {} {
 	 {"Spell checking"   	cmd {SpellChecking}}
 	 {"Glossary"		cmd {EditGlossary} -bind "Ctrl-k"}
 	 {"Speakers"		cascade {
-	    {"Find speaker"		cmd {::speaker::find}}	
-	    {"Import from file..." 	cmd {::speaker::import}}
-	    {"Remove unused speakers" 		cmd {::speaker::purge}}
+	     {"Find speaker"		cmd {::speaker::find}}	
+	     {"Import from file..." 	cmd {::speaker::import}}
+	     {"Remove unused speakers" 		cmd {::speaker::purge}}
 	 }}
 	 {"Topics"		cascade {
 	    {"Find topic"		cmd {::topic::find}}	
 	    {"Import from file..." 	cmd {::topic::import}}
 	    {"Remove unused topics" 		cmd {::topic::purge}}
+	    {""}
+ 	    {"Automatic import from selected file"	check v(importTopics)}
 	 }}
 	 {""}
 	 {"Insert event..."		cascade {
 	    {"Isolated noise"	cmd {CreateEvent "b" "noise" "instantaneous" 1}  -bind "Ctrl-d"}
-	    {"Overlapping noise"	cmd {CreateEvent "b" "noise" "previous" 1}}
-	    {"Pronounce"		cmd {CreateEvent "" "pronounce" "previous" 1} -bind "Alt-equal"}
-	    {"Language"			cmd {CreateEvent "en" "language" "previous" 1}}
-	    {"Lexical"			cmd {CreateEvent "" "lexical" "previous" 1}}
-	    {"Comment"			cmd {CreateEvent "" "comment"}}
+	    {"Overlapping noise"	cmd {CreateAutoEvent "b" "noise" "previous" 1}}
+	    {"Pronounce"		cmd {CreateAutoEvent "" "pronounce" "previous" 1} -bind "Alt-equal"}
+	    {"Language"			cmd {CreateAutoEvent "en" "language" "previous" 1}}
+	    {"Lexical"			cmd {CreateAutoEvent "" "lexical" "previous" 1}}
+	    {"Comment"			cmd {CreateAutoEvent "" "comment"}}
+	    {"Named Entities"                 cmd {CreateAutoEvent "" "entities"} -bind "Ctrl-e"} 
 	 }}
       }}
       {"Signal" -underline 0	cascade {
@@ -284,7 +303,7 @@ proc InitMenus {} {
 	 {""}
 	 {"Control panel"	cmd {CreateGainFrame}}
       }}
-      {"Segmentation" -underline 0	cascade {
+      {"Segmentation" -underline 2	cascade {
 	 {"Move to..."	cascade {
 	    {"Next synchro"	cmd {TextNextSync +1} -acc "Down"}
 	    {"Previous synchro"	cmd {TextNextSync -1} -acc "Up"}
@@ -302,12 +321,12 @@ proc InitMenus {} {
 	 {"Insert background"	cmd {CreateBackground}}
 	 {""}
 	 {"Create turn..."	cmd {ChangeSegType Turn} -bind "Ctrl-t"}
-	 {"Create section..."	cmd {ChangeSegType Section} -bind "Ctrl-e"}
+	 {"Create section..."	cmd {ChangeSegType Section} -bind "Ctrl-r"}
 	 {"Edit turn attributes..."	cmd {::turn::edit} -bind "Ctrl-Alt-t"}	
 	 {"Edit section attributes..."	cmd {::section::edit}}	
 	 {""}
 	 {"Move breakpoint"   cmd { tk_messageBox -type ok -message "Just click on the segment boundary with central button (or control-click with left button) and drag it to the new position! Use shift modifier for a forced move."}}
-	 {"Delete breakpoint"   cmd { DeleteSegment } -bind "Shift-BackSpace"}
+	 {"Delete breakpoint"   cmd { DeleteSegment } -bind "Ctrl-BackSpace"}
       }}
       {"Options" -underline 0	cascade {
 	 {"General..."		cmd {ConfigureGeneral}}
@@ -318,15 +337,17 @@ proc InitMenus {} {
 	    {"Edit pronounce list..."	cmd {ConfEventName "pronounce" "Pronounce"}}
 	    {"Edit lexical list..."	cmd {ConfEventName "lexical" "Lexical"}}
 	    {"Edit language list..."	cmd {ConfEventName "language" "Language"}}
+	    {"Edit named entities list..."	cmd {ConfEventName "entities" "Named Entities"}} 
 	 }}
 	 {"Display" -underline 0	cascade {
-	    {"Text editor"	check v(view,.edit) -command {SwitchTextFrame}}
-	    {"Command buttons"	check v(view,.cmd) -command {SwitchFrame .cmd  -before .snd}}
-	    {"Second signal view"	check v(view,.snd2) -command {SwitchSoundFrame .snd2}}
+	    {"Text editor"	check v(view,.edit) -command {SwitchTextFrame} -bind "F2"}
+	    {"NE buttons"  check v(view,.edit.ne) -command {SwitchNEFrame .edit.ne} -bind "F3"}
+	    {"Command buttons"	check v(view,.cmd) -command {SwitchFrame .cmd  -after .edit} -bind "F4"}
+	    {"First signal view"	check v(view,.snd) -command {SwitchFrame .snd} -bind "F5"} 
+	    {"Second signal view"	check v(view,.snd2) -command {SwitchFrame .snd2} -bind "F6"}
+	    {"Messages"  check v(view,.msg) -command {SwitchFrame .msg -side bottom} -bind "F7"}
 	    {"Smart segmentation display"	check v(hideLevels) -command {UpdateSegmtView}}
 	    {"Colorize speaker segments"	check v(colorizeSpk) -command {ColorizeSpk}}
-	    {""}
-	    {"Open segmentation file..." 	cmd {OpenSegmt}}
 	 }}
 	 {"Fonts"	cascade {
 	    {"Text"		cmd {set v(font,text)  [ChooseFont text] }}
@@ -336,6 +357,7 @@ proc InitMenus {} {
 	    {"Messages"		cmd {set v(font,mesg)  [ChooseFont mesg] }}
 	    {"Lists"		cmd {set v(font,list)  [ChooseFont list] }}
 	    {"Axis"		cmd {set v(font,axis)  [ChooseFont axis] }}
+	    {"NE buttons"      cmd {set v(font,NEbutton) [ChooseFont NEbutton];UpdateNEFrame .edit.ne }} 
 	 }}
 	 {"Colors..."		cmd {ConfigureColors}}
 	 {"Bindings..."		cmd {ConfigureBindings}}
@@ -345,7 +367,7 @@ proc InitMenus {} {
  	 {"Save configuration as..."	cmd {SaveOptions as}}
       }}
       {"Help" -underline 0 cascade {
-	 {"About..."		cmd {ViewHelp "Index"}}
+	 {"About..."		cmd {ViewHelp "Index"} -bind "F1"}
 	 {""}
 	 {"Presentation"	cmd {ViewHelp "Presentation"}}
 	 {"Main features"	cmd {ViewHelp "Main features"}}
