@@ -1,9 +1,11 @@
-# Copyright (C) 1998, DGA - part of the Transcriber program
+# RCS: @(#) $Id: Segmt.tcl,v 1.7 2003/08/27 14:55:39 barras Exp $
+
+# Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
 
 ################################################################
 
-proc CreateSegmentWidget {wavfm seg args} {
+proc CreateSegmentWidget {wavfm seg name args} {
    global v
 
    set f [winfo parent $wavfm].$seg
@@ -11,7 +13,8 @@ proc CreateSegmentWidget {wavfm seg args} {
       eval {segmt $f -seg v(trans,$seg) -bd 1 -padx 1m -pady 1 -font trans -bg $v(color,bg) -time ::Synchro::time} $args
       setdef v(view,$f) 1
       if {$v(view,$f)} {
-	 pack $f -fill x -padx 10 -after $wavfm
+	 #pack $f -fill x -padx 10 -after $wavfm
+	 pack $f -fill x -padx 10 -before [winfo parent $wavfm].a
       }
 
       # Segment selection with B1
@@ -20,19 +23,21 @@ proc CreateSegmentWidget {wavfm seg args} {
       bind $f <ButtonRelease-1>  [list EndSegmentSelect $seg %X %y]
       bind $f <Shift-Button-1>   [list SegmentSelect $wavfm $seg 1 %X %y]
 
-      # Segment boundary move with B2 or Control-B1
-      bind $f <Button-2>   [list SegmentMove $wavfm $seg 0 %X %y]
-      bind $f <B2-Motion>  [list SegmentMove $wavfm $seg 1 %X %y]
-      bind $f <ButtonRelease-2>  [list EndSegmentMove $seg]
-      bind $f <Control-Button-1>   [list SegmentMove $wavfm $seg 0 %X %y]
-      bind $f <Control-B1-Motion>  [list SegmentMove $wavfm $seg 1 %X %y]
-      bind $f <Control-ButtonRelease-1>  [list EndSegmentMove $seg]
+      # Segment boundary move with B2 or Control-B1; forced move with Shift-
+      foreach F {0 1} S {"" "Shift-"} {
+	 bind $f <${S}Button-2>   [list SegmentMove $wavfm $seg 0 %X %y $F]
+	 bind $f <${S}B2-Motion>  [list SegmentMove $wavfm $seg 1 %X %y $F]
+	 bind $f <${S}ButtonRelease-2>  [list EndSegmentMove $seg]
+	 bind $f <${S}Control-Button-1>   [list SegmentMove $wavfm $seg 0 %X %y $F]
+	 bind $f <${S}Control-B1-Motion>  [list SegmentMove $wavfm $seg 1 %X %y $F]
+	 bind $f <${S}Control-ButtonRelease-1>  [list EndSegmentMove $seg]
+      }
       
       # Contextual menu with B3
       bind $f <Button-3>  [list tk_popup $v($wavfm,menu) %X %Y]
       set menu [$v($wavfm,menu) entrycget [Local "Display"] -menu]
       add_menu $menu [subst {
-	 {"$seg" check v(view,$f) -command {SwitchSegmtView $wavfm}}
+	 {"$name" check v(view,$f) -command {SwitchSegmtView $wavfm}}
       }]
 
       lappend v($wavfm,seglist) $f
@@ -46,32 +51,46 @@ proc SwitchSegmtView {wavfm} {
    global v
 
    foreach f $v($wavfm,seglist) {
-      if {$v(view,$f)} {
-	 pack $f -fill x -padx 10 -after $wavfm
-      } else {
-	 pack forget $f
-      }
+     if {$v(view,$f)} {
+       #pack $f -fill x -padx 10 -after $wavfm
+       pack $f -fill x -padx 10 -before [winfo parent $wavfm].a
+     } else {
+       pack forget $f
+     }
    }
 }
 
+proc UpdateSegmtView {{mode initial}} {
+  global v
+
+  if {$v(hideLevels)} {
+    foreach lvl {seg1 seg2 bg} {
+      foreach s [array names v view,*.$lvl] {
+	if {[llength $v(list,$lvl)] > 1} {
+	  if {$mode == "initial" || [llength $v(list,$lvl)] == 2} {
+	    set v($s) 1
+	  }
+	} elseif {$mode == "initial"} {
+	  set v($s) 0
+	}
+      }
+    }
+    catch {
+      foreach w $v(wavfm,list) {
+	SwitchSegmtView $w
+      }
+    }
+  }
+}
+
 proc CreateAllSegmentWidgets {} {
    global v
 
    foreach wavfm $v(wavfm,list) {
-      CreateSegmentWidget $wavfm seg0 -fg $v(color,fg-sync) -full $v(color,bg-sync) -height 2 -high $v(color,hi-sync)
-      CreateSegmentWidget $wavfm seg1 -fg $v(color,fg-turn) -full $v(color,bg-turn) -tiny {fixed 8}
-      CreateSegmentWidget $wavfm seg2 -fg $v(color,fg-sect) -full $v(color,bg-sect)
-      CreateSegmentWidget $wavfm bg   -fg $v(color,fg-back) -full $v(color,bg-back) -empty $v(color,bg)
-   }
-}
-
-proc DestroySegmentWidget {wavfm seg} {
-   global v
-
-   set f [winfo parent $wavfm].$seg
-   if [winfo exists $f] {
-      destroy $f
-      lsuppress v($wavfm,sync) $f
+      CreateSegmentWidget $wavfm bg   "Background" -fg $v(color,fg-back) -full $v(color,bg-back) -empty $v(color,bg)
+      CreateSegmentWidget $wavfm seg2 "Sections" -fg $v(color,fg-sect) -full $v(color,bg-sect)
+      CreateSegmentWidget $wavfm seg1 "Turns" -fg $v(color,fg-turn) -full $v(color,bg-turn) -tiny {fixed 8}
+      CreateSegmentWidget $wavfm seg0 "Transcription" -fg $v(color,fg-sync) -full $v(color,bg-sync) -height 2 -high $v(color,hi-sync)
    }
 }
 
@@ -88,6 +107,34 @@ proc DestroySegmentWidgets {} {
    }
 }
  
+proc DestroyLabels {} {
+  global v
+
+  # destroy label segment widgets and associated list
+  foreach seg0 [array names v trans,lbl*] {
+    set seg [lindex [split $seg0 ,] 1]
+    foreach wavfm $v(wavfm,list) {
+      set f [winfo parent $wavfm].$seg
+      if [winfo exists $f] {
+	destroy $f
+	lsuppress v($wavfm,sync) $f
+	lsuppress v($wavfm,seglist) $f
+	catch {unset v(view,$f)}
+      }
+    }
+    unset v(trans,$seg)
+  }
+  set v(labelNames) {}
+  # destroy menu entries
+  foreach wavfm $v(wavfm,list) {
+    catch {
+      set menu [$v($wavfm,menu) entrycget [Local "Display"] -menu]
+      set index [$menu index [Local "Transcription"]]
+      $menu delete [incr index] end
+    }
+  }
+}
+
 ################################################################
 
 # Empty segmentations
@@ -177,6 +224,9 @@ proc JoinSegmt {segmt nb1 {mode ""} {txt ""}} {
       set txt [concat [lindex $seg1 2] [lindex $seg2 2]]
    }
    set seg [list [lindex $seg1 0] [lindex $seg2 1] $txt]
+   if {$segmt == "seg1" && $v(colorizeSpk)} {
+     lappend seg [ColorMap $txt]
+   }
    set v(trans,$segmt) [lreplace $v(trans,$segmt) $nb1 $nb2 $seg]
    set v(list,$segmt) [lreplace $v(list,$segmt) $nb2 $nb2]
 }
@@ -195,6 +245,10 @@ proc SplitSegmt {segmt nb pos txt1 txt2 id} {
    }
    set seg1 [list [lindex $seg 0] $pos $txt1]
    set seg2 [list $pos [lindex $seg 1] $txt2]
+   if {$segmt == "seg1" && $v(colorizeSpk)} {
+     lappend seg1 [ColorMap $txt1]
+     lappend seg2 [ColorMap $txt2]
+   }
    set v(trans,$segmt) [lreplace $v(trans,$segmt) $nb $nb $seg1 $seg2]
    set v(list,$segmt) [linsert $v(list,$segmt) [expr $nb+1] $id]
 }
@@ -272,6 +326,8 @@ proc GetPosAfterScroll {wavfm X} {
    set pos [GetClickPos $wavfm $X scroll]
    if {$scroll != 0} {
       ScrollTime $wavfm scroll $scroll units
+      # Tempo for smooth scrolling
+      after 50
       # Get new position after scroll
       set pos [GetClickPos $wavfm $X scroll]
    }
@@ -299,15 +355,18 @@ proc SegmentSelect {wavfm segmt extend X y} {
    PauseAudio
    set newbeg $beg
    set newend $end
+   set mode "BEGIN"
    if {$extend && [GetSelection oldbeg oldend]} {
       if {($oldbeg < $beg) && ($oldend > $end)} {
 	 if {$oldend-$beg < $end-$oldbeg} {
 	    set newbeg $oldbeg
+	    set mode "END"
 	 } else {
 	    set newend $oldend
 	 }
       } elseif {$oldbeg < $beg} {
 	 set newbeg $oldbeg
+	 set mode "END"
       } elseif {$oldend > $end} {
 	 set newend $oldend
       }
@@ -316,7 +375,7 @@ proc SegmentSelect {wavfm segmt extend X y} {
    }
    ViewSelection $beg $end
    SetSelection $newbeg $newend
-   ViewSelection $beg $end
+   ViewSelection $beg $end $mode
 }
 
 proc EndSegmentSelect {segmt X y} {
@@ -331,80 +390,120 @@ proc EndSegmentSelect {segmt X y} {
 
 # Events bindings for modification of segment boundaries
 
-proc SegmentMove {wavfm segmt extend X y} {
+proc SegmentMove {wavfm segmt extend X y {force 0}} {
    global v
 
+   set reso 0.001
    set pos [GetPosAfterScroll $wavfm $X]
    set epsilon [expr 10.0*$v($wavfm,size)/[winfo width $wavfm]]
+   if {$epsilon < $reso} {set epsilon $reso}
    if {!$extend} {
       # Be sure we start close enough from existing boundary (10 pixels)
-      set nb1 [GetSegmentFromPos $segmt $pos beg end ov]
-      if {$nb1 < 0} return
+      set nb [GetSegmentFromPos $segmt $pos beg end ov]
+      if {$nb < 0} return
       if {$ov == 1} {
 	 set f [winfo parent $wavfm].$segmt
-	 set nb1 [expr $nb1 + ($y > [winfo height $f]/2)]
-	 set beg [GetSegmtField $segmt $nb1 -begin]
-	 set end [GetSegmtField $segmt $nb1 -end]
+	 set nb [expr $nb + ($y > [winfo height $f]/2)]
+	 set beg [GetSegmtField $segmt $nb -begin]
+	 set end [GetSegmtField $segmt $nb -end]
       }
       if {abs($pos-$beg) > $epsilon && abs($pos-$end) > $epsilon } return
 
       # Choose the right side of segment
       if {$pos-$beg < $end-$pos} {
-	 incr nb1 -1
+	 incr nb -1
       }
-
+      set v(moved_id) {}
    } else {
       if {![info exists v(segmt,move)]} return
       # Keep moving same boundary during mouse motion
-      set nb1 $v(segmt,move)
+      set nb $v(segmt,move)
    }
 
-   # Be sure we are not the first or last segment
-   set nb2 [expr $nb1+1]
-   if {$nb1 < 0 || $nb2 >= [GetSegmtNb $segmt]} return
+   set nb1 $nb
+   set pos1 $pos
+   set moves {}
+   set id ""
+   while {1} {
+      # Be sure we are not the first or last segment
+      if {$nb < 0 || [expr $nb+1] >= [GetSegmtNb $segmt]} return
+      
+      # Get boundaries
+      if {$id == ""} {
+	 set id [GetSegmtField $segmt $nb -endId]
+      }
+      if [catch {expr double($id)}] {
+	 Synchro::GetBoundaries $id center left right leftIds rightIds
+	 if {$left == "" || $right == ""} return
+      } else {
+	 set center $id
+	 set left   [GetSegmtField $segmt $nb -begin]
+	 set right  [GetSegmtField $segmt [expr $nb+1] -end]
+	 set id     ""
+ 	 set rightIds [set leftIds {}]
+      }
+      lappend moves [list $nb $id $pos $center $left $right $leftIds $rightIds]
+      #puts "$nb $id $left $center $right => $pos"
 
-   set id [GetSegmtField $segmt $nb1 -endId]
-   if [catch {expr double($id)}] {
-      Synchro::GetBoundaries $id center left right
-   } else {
-      set center $id
-      set left   [GetSegmtField $segmt $nb1 -begin]
-      set right  [GetSegmtField $segmt $nb2 -end]
-      set id     ""
+      # Keep minimal apparent size for both segments
+      if {$pos < $center && $pos-$left < $epsilon} {
+	 if {!$force} return
+	 if {$id != ""} {
+	    set pos [expr $pos - $epsilon]
+	    set id $Synchro::value($left)
+	    continue
+	 }
+      } elseif {$pos > $center && $right-$pos < $epsilon} {
+	 if {!$force} return
+	 if {$id != ""} {
+	    set pos [expr $pos + $epsilon]
+	    set id $Synchro::value($right)
+	    continue
+	 }
+      }
+      break
    }
 
-   # Keep minimal apparent size for both segments
-   if {($pos < $center && $pos-$left < $epsilon) 
-       || ($pos > $center && $right-$pos < $epsilon)} return
-
    # Really do it
    if {!$extend} {
       PauseAudio
       set v(segmt,move) $nb1
-      # Register old position for undo
+      # Register old position and floating state for undo
       if {$id != ""} {
-	 DoModif "MOVE $id $center"
+	 DoModif "MOVE $id $center [Synchro::getElastic $id]"
       } else {
 	 DoModif "MOVE"
       }
    }
-   if {$id == ""} {
-      SetSegmtField $segmt $nb1 -end $pos
-      SetSegmtField $segmt $nb2 -begin $pos
-   } else {
-      Synchro::ModifyTime $id $pos
+   foreach move $moves {
+      foreach {nb id pos center left right leftIds rightIds} $move break
+      if {$id == ""} {
+	 SetSegmtField $segmt $nb -end $pos
+	 SetSegmtField $segmt [expr $nb+1] -begin $pos
+      } else {
+	 Synchro::ModifyTime $id $pos
+	 Synchro::setElastic $id 0
+	 Synchro::ModifyElastic $leftIds $center $pos $left
+	 Synchro::ModifyElastic $rightIds $center $pos $right
+	 # Keep the list of all moved boundary ids
+	 foreach i [concat $id $leftIds $rightIds] {
+	   if {[lsearch -exact $v(moved_id) $i] < 0} {
+	     lappend v(moved_id) $i
+	   }
+	 }
+      }
    }
-   SetSelection $pos $pos
+   SetSelection $pos1 $pos1
 }
 
 proc EndSegmentMove {segmt} {
    global v
 
    catch {
-      set id [GetSegmtField $segmt $v(segmt,move) -endId]
-      if [catch {expr double($id)}] {
+      foreach id $v(moved_id) {
 	 Synchro::UpdateTimeTags $id
       }
+      unset v(moved_id)
       unset v(segmt,move)
    }
 }
@@ -417,7 +516,7 @@ proc EndSegmentMove {segmt} {
 proc SetCurrentSegment {nb} {
    global v
 
-   if {$nb < 0 || $nb >= [GetSegmtNb seg0]} return
+   if {$nb < 0 || $nb >= [GetSegmtNb seg0] || [info exists v(demo)]} return
    set v(segmt,curr) $nb
 
    # Highlight current segment in text editor
@@ -439,8 +538,13 @@ proc SetCurrentSegment {nb} {
        || [$t compare insert > "$last"]} {
       set t $v(tk,edit)
       catch {
-	 $t see "$first"
-	 tkTextSetCursor $t "$last lineend"
+	 if {$v(preferedPos) == "begin"} {
+	    $t see "$last"
+	    tkTextSetCursor $t "$first linestart"
+	 } else {
+	    $t see "$first"
+	    tkTextSetCursor $t "$last lineend"
+	 }
       }
    }
    ViewAroundText
@@ -453,8 +557,8 @@ proc SetCurrentSegment {nb} {
       # Select corresponding part of signal
       set play [IsPlaying]
       if {$play} {PauseAudio}
-      ViewSelection $beg $end
       SetSelection $beg $beg
+      ViewSelection $beg $end "BEGIN"
       if {$play} {Play}
    }
 }
@@ -622,6 +726,10 @@ proc SplitTransTags {level child vals} {
       if {$fidelity != ""} {
 	 lappend atts "fidelity" $fidelity
       }
+      set channel [lindex $vals 3]
+      if {$channel != ""} {
+	 lappend atts "channel" $channel
+      }
       set name [::speaker::name $speaker]
    } else {
       set insertButton InsertSectionButton
@@ -799,6 +907,7 @@ proc InsertSegment {} {
    # let cursor move further inside segment if playback in course
    update
    SetCurrentSegment [incr nb]
+   tkTextSetCursor $v(tk,edit) "insert linestart"
 }
 
 ################################################################
@@ -915,6 +1024,7 @@ proc ChangeSegType {category {nb {}}} {
 	    if [catch {
 	       set tur_atts [::turn::choose $spk]
 	    }] return
+	    #puts $tur_atts
 	    set turn [SplitTransTags 1 $sync $tur_atts]
 	 } else {
 	    #::turn::edit
@@ -931,6 +1041,7 @@ proc ChangeSegType {category {nb {}}} {
       }
    }
    DoModif "TYPE"
+   UpdateSegmtView modified
 }
 
 ################################################################
