@@ -1,4 +1,4 @@
-# RCS: @(#) $Id: Signal.tcl,v 1.6 2003/08/25 16:54:17 barras Exp $
+# RCS: @(#) $Id: Signal.tcl,v 1.14 2005/01/11 12:24:20 mantam Exp $
 
 # Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
@@ -37,10 +37,10 @@ proc EmptySignal {{mode "reset"}} {
    }
    set v(sig,name) ""
    UpdateShortName
-   set v(sig,desc) "Signal:\tnone\nDuration:\t[Tim2Str $v(sig,len)]\n"
+    set v(sig,desc) "Signal:\tnone\n[Local Duration:]\t[Tim2Str $v(sig,len)]\n"
    catch {$v(shape,cmd) destroy}
    set v(shape,cmd) ""
-   ConfigAllWavfm
+   ConfigAllWavfm $mode
    if {$mode == "reset"} {
      SetSelection 0 0
      #$v(tk,gain) set 0
@@ -53,7 +53,9 @@ proc EmptySignal {{mode "reset"}} {
 proc Signal {name {mode "reset"}} {
    global v
 
+   set playing [IsPlaying]
    # Forget previous signal, open new sound and set values
+  
    EmptySignal $mode
    set sound [OpenSound $name]
    set v(sig,len) [$sound length -unit seconds]
@@ -61,7 +63,7 @@ proc Signal {name {mode "reset"}} {
    set v(sig,name) $name
    UpdateShortName
    set stereo [lindex {"" "mono" "stereo" "" "quad"}  [$sound cget -channels]]
-   set v(sig,desc) "Signal:\t$name\nDuration:\t[Tim2Str $v(sig,len)]\nFormat:\t[$sound cget -format] [format %g [expr [$sound cget -frequency]/1000.0]] kHz $stereo"
+   set v(sig,desc) "Signal:\t$name\n[Local Duration:]\t[Tim2Str $v(sig,len)]\nFormat:\t[$sound cget -format] [format %g [expr [$sound cget -frequency]/1000.0]] kHz $stereo"
 
    # Add on top of list of sound paths
    if {! $v(sig,remote)} {
@@ -74,7 +76,7 @@ proc Signal {name {mode "reset"}} {
    # for files longer than 20 sec (by default)
    # If shape inactive, should forbid global view of signal.
    # If v(shape,bg) is true, calculation is done in background.
-   # For remote sounds, no background process implemented.
+
    set v(shape,cmd) ""
    if {$v(shape,wanted) && $v(sig,len) >= $v(shape,min)} {
       if {! $v(sig,remote)} {
@@ -92,20 +94,28 @@ proc Signal {name {mode "reset"}} {
 	       ShapeAbort
 	       
 	       # launch sub-process
-	      set bg [open [list "|" [info nameofexecutable] [file join $v(path,tcl) BgShape.tcl] $name $shapeName $v(sig,rate) $v(sig,channels) $v(sig,header)]]
+               if {$::tcl_platform(os) == "Darwin"} {
+                  set tclsh "tclsh"
+               } else {
+                  set tclsh [info nameofexecutable]
+               }
+	       set bg [open [list "|" $tclsh [file join $v(path,tcl) BgShape.tcl] $name $shapeName $v(sig,rate) $v(sig,channels) $v(sig,header)]]
 	       fileevent $bg readable [list ShapeDone $bg $sound $name $shapeName]
 	       set v(shape,bgchan) $bg
 	       
 	       # User-friendly info box about current process
 	       toplevel .shp -cursor watch
 	       wm title .shp "Shape info"
-	       label .shp.l -text "Currently computing global shape for signal\n[file tail $name]"
-	       button .shp.b -text "Abort" -command ShapeAbort
+		label .shp.l -text "[Local "Currently computing global shape for signal"]\n[file tail $name]"
+           if {$::tcl_platform(platform) != "windows"} {
+	       button .shp.b -text [Local "Abort"] -command ShapeAbort
+               pack .shp.b -padx 3m -pady 2m
+           }             
 	       pack .shp.l -fill both -expand true -padx 3m -pady 2m
-	       pack .shp.b -padx 3m -pady 2m
+	       
 	       
 	    } else {
-	       DisplayMessage "Computing signal shape. Please wait..."
+	       DisplayMessage [Local "Computing signal shape. Please wait..."]
 	       update
 	       $sound shape $shp -format MULAW
 	       $shp write $shapeName -fileformat WAV
@@ -118,9 +128,9 @@ proc Signal {name {mode "reset"}} {
       }
    }
    if {$v(shape,cmd) != ""} {
-      append v(sig,desc) "\nShape:\tyes"
+      append v(sig,desc) "\n[Local "Shape: yes"]"
    } else {
-      append v(sig,desc) "\nShape:\tno"
+      append v(sig,desc) "\n[Local "Shape: no"]"
    }
 
    # Synchronize end times between signal and transcription
@@ -143,13 +153,17 @@ proc Signal {name {mode "reset"}} {
       }
    }
 
-   ConfigAllWavfm
+   ConfigAllWavfm $mode
    update
 
    if {$mode != "switch"} {
      MW_AddFile $v(sig,name)
    }
 
+   if {$mode == "switch" && $playing} {
+     Play
+   }
+
    if {$v(play,auto)} {
       PlayFromBegin
    }
@@ -160,11 +174,15 @@ proc Signal {name {mode "reset"}} {
 proc ShapeDone {channel sound sigName shapeName} {
    global v
 
-   set res [string trim [read $channel]]
-   if {![catch {
-      close $channel
-   } err]
-       && $v(sig,cmd) == $sound
+   fileevent $channel readable {}
+    catch {
+        exec kill -9 [pid $channel]
+    }
+    set res [string trim [read $channel]]
+   catch {
+     close $channel
+   } 
+   if {$v(sig,cmd) == $sound
        && $v(sig,name) == $sigName
        && $res == $shapeName
     } {
@@ -172,12 +190,12 @@ proc ShapeDone {channel sound sigName shapeName} {
       foreach wavfm $v(wavfm,list) {
 	$wavfm config -shape $v(shape,cmd)
       } 
-      DisplayMessage "Signal shape is now available!"
-      append v(sig,desc) "\nShape:\tyes"
+      DisplayMessage [Local "Signal shape is now available!"]
+      append v(sig,desc) "\n[Local "Shape: yes"]"
    } else {
       puts "error $res"; puts $err
-      DisplayMessage "Signal shape not available, sorry..."
-      append v(sig,desc) "\nShape:\tno"
+      DislayMessage [Local "Signal shape not available, sorry..."]
+      append v(sig,desc) "\n[Local "Shape: no"]"
    }
    unset v(shape,bgchan)
    destroy .shp
@@ -210,7 +228,7 @@ proc LookForShape {sigName} {
 
    # Search for an existing matching shape
    # (in default dir, shp sub-dir or signal dir)
-   foreach path [concat $v(path,shape) "shp ../shp ."] {
+   foreach path [concat [file join $v(path,shape)] "shp ../shp ."] {
       # Relative paths are relative to signal path
       set path [file join [file dirname $sigName] $path]
       set shape [file join $path $base.$ext]
@@ -306,8 +324,12 @@ proc OpenAudioFile {{mode "reset"}} {
 	 set audiopen ""
       }
    } else {
+	   set upperExt [string toupper $v(ext,snd)]
+	   set allExt [concat $v(ext,snd) $upperExt]
       set types [subst {
-	 {"Audio files" {$v(ext,snd)}}
+	      
+	 {"Audio files" {$allExt} }
+	 
 	 {"All files"   {*}}
       }]
       set path [lindex $v(path,sounds) 0]
@@ -317,11 +339,23 @@ proc OpenAudioFile {{mode "reset"}} {
       if {![file isdir $path]} {
 	 set path [pwd]
       }
-      set audiopen [tk_getOpenFile -filetypes $types \
-		  -initialdir $path -title "Open audio file"]
+      # support multiple simultaneous file open (Tcl/Tk>=8.4) for MultiWav mode
+      if {[info tclversion] >= 8.4 && $mode == "add"} {
+	set audiopen [tk_getOpenFile -filetypes $types -multiple 1\
+			-initialdir $path -title "Open audio file"]
+      } else {
+	set audiopen [tk_getOpenFile -filetypes $types \
+			-initialdir $path -title "Open audio file"]
+      }
    }
    if {$audiopen != ""} {
-      Signal $audiopen $mode
+      if {[info tclversion] >= 8.4 && $mode == "add"} {
+	foreach audio $audiopen {
+	  Signal $audio $mode
+	}
+      } else {
+	Signal $audiopen $mode
+      }
       UpdateFilename
    }
    unset audiopen
@@ -381,10 +415,10 @@ proc ConfigureAudioFile {} {
    checkbutton $g.shp -text [Local "Compute low-resolution shape for long sound files"] -variable v(shape,wanted) -anchor w -padx 3m -command "FrameState $g.1 \$v(shape,wanted)"
    set h [frame $g.1]
    pack $g.shp $g.1 -side top -fill x
-   if {$::tcl_platform(platform) != "windows"} {
+#"   if {$::tcl_platform(platform) != "windows"} {
       checkbutton $h.bg -text [Local "Do shape computation in background"] -variable v(shape,bg) -anchor w -padx 3m
-      pack $h.bg
-   }
+#"      pack $h.bg
+#"   }
    EntryFrame $h.en1 "Store signal shapes in" v(path,shape)
    catch {
     pack $h.bg $h.en1 -side top -fill x
@@ -503,7 +537,6 @@ proc OpenSound {name {player 0}} {
       # Open audio file on server with format options
       eval $sound "sound -file [list $name] -channels $v(sig,channels) -frequency $v(sig,rate) -skiphead $v(sig,header) -guessproperties 1"
    }
-   set v(sig,open) "1"
    return $sound
 }
 
