@@ -1,4 +1,4 @@
-# RCS: @(#) $Id: Spelling.tcl,v 1.6 2003/08/25 16:54:17 barras Exp $
+# RCS: @(#) $Id: Spelling.tcl,v 1.11 2004/11/22 16:10:23 mantam Exp $
 
 # Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
@@ -9,12 +9,13 @@
 proc SpellChecking {} {
    global v
 
-   if {$::tcl_platform(platform) == "windows"} {
+   if {$::tcl_platform(os) == "Darwin"} {
      tk_messageBox -type ok -icon error -message \
- 	 [Local "Sorry, spell checker not available on this platform"]
+        [Local "Sorry, spell checker not available on this platform"]
+        return
    }
 
-   OpenIspell
+   OpenAspell
 
    if ![info exists v(tk,edit)] return
    set t $v(tk,edit)
@@ -155,41 +156,49 @@ proc SpellUpdate {} {
 }
 
 proc SpellWindow {} {
-   global v
+    global v
 
-   set w .spell
-   if ![winfo exists $w] {
-      toplevel $w
-      wm title $w [Local "Spell checking"]
-      wm protocol $w WM_DELETE_WINDOW { SpellClose }
 
-      frame $w.left -relief raised -bd 1
 
-      set v(spell,ent) [EntryFrame $w.left.wrd "Word" v(spell,new)]
-      #$w....lab conf -width 10 -anchor w
 
-      set v(spell,lst) [ListFrame $w.left.lst ""]
-      $v(spell,lst) conf -height 6
-      bind $v(spell,lst) <ButtonRelease-1> SpellChooseGuess
+    set w .spell
+    if ![winfo exists $w] {
+	toplevel $w
+	wm title $w [Local "Spell checking"]
+	wm protocol $w WM_DELETE_WINDOW { SpellClose }
+	
+	if {$::tcl_platform(platform) == "windows"} {
+	      wm attributes $w -topmost 1 
+      }
 
-      frame $w.but -relief raised -bd 1
-      button $w.but.add -text [Local "Add to dictionnary"] -command SpellAccept
-      button $w.but.ignore -text [Local "Ignore"] -command SpellIgnore
-      button $w.but.replace -text [Local "Replace"] -command SpellReplace
-      button $w.but.close -text [Local "Close"] -command SpellClose
-      pack $w.but.add $w.but.ignore $w.but.replace $w.but.close -side top \
-	  -fill x -expand 1 -padx 2m -pady 1m
-
-      #button $w.but.close -text "Close" -command [list wm withdraw $w]
-      #pack $w.but.next $w.but.repl $w.but.repa $w.but.close -side left \
-	\#  -expand 1 -padx 2m -pady 1m
-
-      pack $w.left $w.but -side left -fill both -expand true
-      focus $v(spell,ent)
-      bind $w <Escape> "tkButtonInvoke $w.but.close"
+	frame $w.left -relief raised -bd 1
+	
+	set v(spell,ent) [EntryFrame $w.left.wrd "Word" v(spell,new)]
+	#$w....lab conf -width 10 -anchor w
+	
+	set v(spell,lst) [ListFrame $w.left.lst ""]
+	$v(spell,lst) conf -height 6
+	bind $v(spell,lst) <ButtonRelease-1> SpellChooseGuess
+	
+	frame $w.but -relief raised -bd 1
+	button $w.but.add -text [Local "Add to dictionnary"] -command SpellAccept
+	button $w.but.ignore -text [Local "Ignore"] -command SpellIgnore
+	button $w.but.replace -text [Local "Replace"] -command SpellReplace
+	button $w.but.close -text [Local "Close"] -command SpellClose
+	pack $w.but.add $w.but.ignore $w.but.replace $w.but.close -side top \
+	    -fill x -expand 1 -padx 2m -pady 1m
+	
+	#button $w.but.close -text "Close" -command [list wm withdraw $w]
+	#pack $w.but.next $w.but.repl $w.but.repa $w.but.close -side left \
+	    \#  -expand 1 -padx 2m -pady 1m
+	
+	pack $w.left $w.but -side left -fill both -expand true
+	focus $v(spell,ent)
+	bind $w <Escape> "tkButtonInvoke $w.but.close"
+	CenterWindow $w
    } else {
-      wm withdraw $w
-      wm deiconify $w
+       wm withdraw $w
+       wm deiconify $w
    }
 }
 
@@ -258,16 +267,16 @@ proc SpellReplace {} {
    SpellLoop
 }
 
-# Close ispell and withdraw spell-checking window
+# Close aspell and withdraw spell-checking window
 proc SpellClose {} {
    global v
 
-   CloseIspell
+   CloseAspell
    $v(tk,edit)-bis tag remove spell 0.0 end
    catch {wm withdraw .spell}
 }
 
-# Check one word via ispell
+# Check one word via aspell
 # Output: 1 if OK, 0 if wrong => word lists in v(spell,miss) / v(spell,guess)
 # called by SpellLoop
 proc SpellOneWrd {wrd} {
@@ -278,12 +287,12 @@ proc SpellOneWrd {wrd} {
       return 1
    }
 
-   # get ispell answer
+   # get aspell answer
    puts $v(spell) $wrd
    set ans [gets $v(spell)]
    gets $v(spell)
 
-   # analyze ispell answer
+   # analyze aspell answer
    set ok 1
    set count 0
    set v(spell,miss) {}
@@ -306,8 +315,8 @@ proc SpellOneWrd {wrd} {
 	 set ok 0
       }
       default {
-	 catch {CloseIspell}
-	 error "Unrecognized ispell answer '$ans' for word '$wrd'"
+	 catch {CloseAspell}
+	 error "Unrecognized aspell answer '$ans' for word '$wrd'"
       }
    }
    # always accept guesses ?
@@ -318,10 +327,10 @@ proc SpellOneWrd {wrd} {
    return $ok
 }
 
-# Open ispell as a subprocess
+# Open aspell as a subprocess
 # Choice of dictionary name not very robust; should be at user option ?
 # Called by SpellOne
-proc OpenIspell {} {
+proc OpenAspell {} {
    global v
 
    if {![info exists v(spell)]} {
@@ -334,19 +343,38 @@ proc OpenIspell {} {
       catch {set lang $::iso639($lang)}
       set lang [string tolower $lang]
       # special test for english
-      # a table [iso639] -> [ispell dictionary names] would be useful here
+      # a table [iso639] -> [aspell dictionary names] would be useful here
       if {$lang == "english"} {
 	 set lang "american"
       } elseif {$lang == ""} {
 	 set lang "default"
       }
 
-      if {[catch {
-	 set chan [open "| ispell -a -d $lang" w+]
-      }]} {
-	 tk_messageBox -type ok -icon error -message [Local "Sorry, couldn't launch spell checker"]\n($err)
-	 return -code return
-      }
+	if {$::tcl_platform(os) == "Darwin"} {
+		  tk_messageBox -type ok -icon error -message [Local "Sorry, ASpell spellchecker doesn't exist on Mac OS yet."]
+		  return -code return
+	}
+
+	if {$::tcl_platform(platform) == "windows"} {
+		if {[catch { set AspellPath [registry get HKEY_LOCAL_MACHINE\\SOFTWARE\\Aspell "Path"]} err ]} {
+            tk_messageBox -type ok -icon error -message [Local "Please, check your Aspell spellchecker installation."]\n($err)
+            OpenURL {aspell.net/win32/}
+		    return -code return
+		} else {
+            set CurPwd [pwd]
+            cd $AspellPath
+            set chan [open "| aspell -d $lang pipe" w+]
+            # return in Transcriber directory
+            cd $CurPwd
+		}
+	}
+
+	if {$::tcl_platform(os) == "Linux"} {
+     	if {[catch { set chan [open "| aspell -d $lang pipe"  w+] }]} {
+		tk_messageBox -type ok -icon error -message [Local "Sorry, couldn't launch spell checker"]\n($err)
+	 	return -code return
+      	}
+	}
       fconfigure $chan -buffering line
       if {[gets $chan res] < 0} {
 	 catch {close $chan} err
@@ -358,13 +386,13 @@ proc OpenIspell {} {
    }
 }
 
-proc CloseIspell {} {
+proc CloseAspell {} {
    global v
 
    # save dictionnary before leaving
    if {[info exists v(spell)]} {
       puts $v(spell) "\#"
-      close $v(spell)
+      catch {close $v(spell)}
       unset v(spell)
    }
 }
