@@ -1,4 +1,4 @@
-# RCS: @(#) $Id: Topic.tcl,v 1.2 2002/07/10 07:36:37 barras Exp $
+# RCS: @(#) $Id: Topic.tcl,v 1.5 2004/07/21 14:46:56 mantam Exp $
 
 # Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
@@ -150,6 +150,9 @@ namespace eval topic {
       catch {destroy $w}
       toplevel $w
       wm title $w [Local "Find topic"]
+      if {$::tcl_platform(platform) == "windows"} {
+	      wm attributes $w -topmost 1 
+      }
       #
       set w1 [frame $w.top -relief raised -bd 1]
       pack $w1 -side top -fill both -expand true
@@ -205,7 +208,7 @@ namespace eval topic {
    }
 
    # Import topics from another XML file
-   proc import {{filename ""}} {
+   proc import {{filename ""} {auto 0}} {
 
       # Get filename through dialog box
       if {$filename == ""} {
@@ -219,11 +222,13 @@ namespace eval topic {
 	 if {$filename == ""} return
       }
 
+      DisplayMessage [format [Local "Importing topics from %s..."] $filename]; update
+
       # Read topic file (with complex trick to avoid collision of ids)
       set pref "import_"
       ::xml::dtd::id::set_prefix $pref
       if {[catch {
-	 set tpctree [::xml::parser::read_file $filename -keepdtd 1]
+	 set tpctree [::xml::parser::read_file $filename -keepdtd 3]
       } err]} {
 	 ::xml::dtd::id::set_prefix ""
 	 return -code error -errorinfo $::errorInfo "Couldn't import topics from $filename"
@@ -235,10 +240,15 @@ namespace eval topic {
       foreach topic [$tpctree getElementChilds "Topic"] {
 	 set name [$topic getAttr "desc"]
 	 if {$name != "" && [search_id $name] == ""} {
-	    lappend tpclst $name
+	    if {$auto} {
+	      create $name
+	    } else {
+	      lappend tpclst $name
+	    }
 	 }
       }
       $tpctree deltree
+      if {$auto} return
       set tpclst [lsort -dictionary $tpclst]
       
       # Select items from list
@@ -263,6 +273,7 @@ namespace eval topic {
 	 foreach i $dial(sel) {
 	    create [lindex $tpclst $i]
 	 }
+	 set ::v(topicFile) $filename
       }
       return 
    }
@@ -435,7 +446,7 @@ namespace eval section {
 
       variable ent
       variable ::topic::nb
-      $ent insert insert "topic\#$nb"
+      $ent insert insert [format $::v(newTopicFmt) $nb]
       $ent select range 0 end
       focus $ent
    }
