@@ -1,4 +1,4 @@
-# RCS: @(#) $Id: Trans.tcl,v 1.10 2003/08/28 13:55:50 barras Exp $
+# RCS: @(#) $Id: Trans.tcl,v 1.22 2005/01/11 12:31:30 mantam Exp $
 
 # Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
@@ -10,10 +10,20 @@ namespace eval trs {
    variable msg "Native XML format"
    variable ext ".trs .xml"
 
+   proc guess {name} {
+     # any XML file with 'Trans' root tag is supposed to be a .trs file
+     set f [open $name]; set magic [read $f 128]; close $f
+     if {[string match "<?xml*" $magic] && [string match "*<!DOCTYPE Trans *" $magic]} {
+       return 1
+     } else {
+       return 0
+     }
+   }
+
    proc import {name} {
       global v
       
-      # Newly created transcription must follow the DTD
+      # Read transcription should follow the original DTD
       ::xml::dtd::xml_read $v(file,dtd)
       set v(trans,root) [::xml::parser::read_file $name -keepdtd 1]
       ::speaker::register
@@ -84,7 +94,7 @@ proc UpdateConvertorMenu {} {
       if {[info command ${format}::export] != ""} {
 	 if {[namespace tail $format] == "cha" && !$v(chatMode)} continue
 	 append_menu "Export" [subst {
-	    {"Export to $msg..."	cmd {SaveTrans as $format}}
+	    {"Export to $msg..."        cmd {SaveTrans as $format}}
 	 }]
       }
    }
@@ -120,11 +130,13 @@ proc LookForLabelFormat {name} {
    return $format
 }
 
-# Open a new independant segmentation file
-proc OpenSegmt {{name ""}} {
+# Open one or severel new independant segmentation files
+proc OpenSegmt {args} {
    global v
 
-   if {$name == ""} {
+   if {$args != ""} {
+     set names $args
+   } else {
      set types [subst {
        {"Label file"   {$v(ext,lbl)}}
        {"All files"   {*}}
@@ -136,11 +148,16 @@ proc OpenSegmt {{name ""}} {
      } else {
        set path $v(trans,path)
      }
-     set name [tk_getOpenFile -filetypes $types -initialdir $path \
-		   -title [Local "Open segmentation file"]]
+     if {[info tclversion] >= 8.4} {
+       set names [tk_getOpenFile -filetypes $types -initialdir $path \
+		     -multiple 1 -title [Local "Open segmentation file"]]
+     } else {
+       set names [list [tk_getOpenFile -filetypes $types -initialdir $path \
+		           -title [Local "Open segmentation file"]]]
+     }
    }
-   if {$name != ""} {
-      if {![file readable $name]} return
+   foreach name $names {
+      if {![file readable $name]} continue
       # Look for a segmentation format matching the file
       set format [LookForLabelFormat $name]
       if {$format == ""} {
@@ -231,6 +248,7 @@ proc ReadTrans {name {soundFile ""} {multiwav {}}} {
 
    DisplayMessage [Local "Cleaning up memory..."]; update
    CloseTrans -nosave
+   EmptyVideo
 
    # Try to open associated sound file as early as possible for non-xml files
    if {$format !="trs"} {
@@ -257,9 +275,18 @@ proc ReadTrans {name {soundFile ""} {multiwav {}}} {
    set v(trans,format) $format
    set v(trans,saved) 0
    set v(trans,path) [file dirname $name]
+   if {$v(importTopics)} {
+     ::topic::import $v(topicFile) 0
+   }
+   if {$v(importSpeakers)} {
+     ::speaker::import $v(speakerFile) 0
+   }
    InitModif
    GetVersion
-   NormalizeTrans
+   if {[set msg [NormalizeTrans]] != ""} {
+     tk_messageBox -message $msg -title [Local "File format checking"] -type ok -icon error
+     puts stderr $msg
+   }
    DisplayTrans
    TraceOpen
 
@@ -289,6 +316,11 @@ proc ReadTrans {name {soundFile ""} {multiwav {}}} {
      eval MW_AddFile $multiwav
      UpdateFilename
    }
+   # trace if the name of the dtd has been updated to do like if the transcription have been modified
+#   if { [info exists ::xml::parser::modifdtd] && $::xml::parser::modifdtd == "1" } {
+#       set v(trans,modif) 1
+#       RegisterAutoSave 
+#   }
 }
 
 proc setChatMode {newMode} {
@@ -328,10 +360,16 @@ proc OpenTransFile {} {
 proc OpenTransOrSoundFile {} {
    global v
 
+   set upperSndExt [string toupper $v(ext,snd)]
+   set allSndExt [concat $v(ext,snd) $upperSndExt]
+
+   set upperTrsExt [concat [string toupper $v(ext,trs)] [string toupper $v(ext,lbl)]]
+   set allTrsExt [concat $v(ext,trs) $v(ext,lbl) $upperTrsExt]
+   
    set types [subst {
       {"All files"   {*}}
-      {"Transcription"   {$v(ext,trs) $v(ext,lbl)}}
-      {"Audio files" {$v(ext,snd)}}
+      {"Transcription"   {$allTrsExt}}
+      {"Audio files" {$allSndExt}}
    }]
    set name [tk_getOpenFile -filetypes $types -initialdir $v(trans,path) \
 		 -title [Local "Open transcription or audio file"]]
@@ -367,10 +405,10 @@ proc WriteTrans {name format} {
    # Try to guess format from extension if not given - unused feature
 #    if {$format == ""} {
 #       foreach format [concat [namespace children convert] "trs"] {
-# 	 if {[info exists ${format}::ext]
-# 	     && [lsearch [set ${format}::ext] $ext] >= 0} {
-# 	    break
-# 	 }
+#          if {[info exists ${format}::ext]
+#              && [lsearch [set ${format}::ext] $ext] >= 0} {
+#             break
+#          }
 #       }
 #    }
    if {[lsearch [set ${format}::ext] $ext] < 0} {
@@ -396,6 +434,7 @@ proc NewTrans {{soundFile ""} {multiwav {}}} {
    global v
 
    if [catch {CloseTrans} err] return
+   EmptyVideo
 
    if {[catch {
       if {$soundFile == "<empty>"} {
@@ -421,6 +460,12 @@ proc NewTrans {{soundFile ""} {multiwav {}}} {
    set v(trans,saved) 0
    SegmtToTrans [list [list $v(sig,min) $v(sig,max) ""]]
    InitEpisode
+   if {$v(importTopics)} {
+     ::topic::import $v(topicFile) 0
+   }
+   if {$v(importSpeakers)} {
+     ::speaker::import $v(speakerFile) 0
+   }
    InitModif
    DisplayTrans
    TraceOpen
@@ -431,6 +476,19 @@ proc NewTrans {{soundFile ""} {multiwav {}}} {
 proc SaveTrans {{as ""} {format ""}} {
    global v
 
+    if { [info exists ::xml::parser::modifdtd] && $::xml::parser::modifdtd == "1" } {
+	set rep [tk_messageBox -message [Local "This file is formated for older version of Transcriber (<1.4.7).\nExport to new format ?\n\n\"yes\" will export to new format and make your file only usable with Transcriber-1.4.7 or higher (recommended).\n\n\"no\" will keep the original format."] -type yesnocancel -icon question]
+	switch $rep {
+	    cancel { return }
+	    yes    {
+		set ::xml::parser::modifdtd 0
+	    }
+	    no {
+		set ::xml::parser::modifdtd 0
+		set ::xml::dtd::name  [file join $v(path,etc)  "$::xml::parser::olddtd"]
+	    }
+	}
+    }
    if {[GetSegmtNb seg0] <= 0} return
    if {$format == ""} {
       set format "trs"
@@ -484,9 +542,24 @@ proc SaveTrans {{as ""} {format ""}} {
    return $name
 }
 
+
 proc SaveIfNeeded {} {
    global v
 
+    if { [info exists ::xml::parser::modifdtd] && $::xml::parser::modifdtd == "1" } {
+	set rep [tk_messageBox -message [Local "This file is formated for older version of Transcriber (<1.4.7).\nExport to new format ?\n\n\"yes\" will export to new format and make your file only usable with Transcriber-1.4.7 or higher (recommended).\n\n\"no\" will keep the original format."] -type yesnocancel -icon question]
+	switch $rep {
+	    cancel { return -code error cancel }
+	    yes    {
+		set ::xml::parser::modifdtd 0
+		if {[SaveTrans]==""} {return -code error cancel}
+	    }
+	    no {
+		set ::xml::parser::modifdtd 0
+		set ::xml::dtd::name  [file join $v(path,etc)  "$::xml::parser::olddtd"]
+	    }
+	}
+    }
    if {[HasModifs]} {
       set answer [tk_messageBox -message [Local "Transcription has been modified - Save before closing?"] -type yesnocancel -icon question]
       switch $answer {
@@ -557,8 +630,11 @@ proc TransInfo {} {
 
 proc NormalizeTrans {} {
    global v
+   set msg ""
 
    if {![info exist v(trans,root)] || $v(trans,root)==""} return
+   DisplayMessage [Local "Checking temporal consistency..."]; update
+
    set episode [$v(trans,root) getChilds "element" "Episode"]
 
    # Contrain sections to be a partition of the episode
@@ -569,10 +645,16 @@ proc NormalizeTrans {} {
        set t2 [$sec getAttr "startTime"]
        if {$t2 < $t1} {
 	 $sec setAttr "startTime" $t1
+	 append msg "shifted start time of overlapping section $t2 -> $t1\n"
        } elseif {$t2 > $t1} {
 	 ::xml::element "Section" [list "type" "nontrans" "startTime" $t1 "endTime" $t2] -before $sec
+	 #append msg "inserted new section between $t1-$t2\n"
        }
        set t1 [$sec getAttr "endTime"]
+       # detect inconsistency in sections
+       if {$t1 <= $t2} {
+	 append msg "WARNING - section interval inconsistency $t2-$t1\n"
+       }
      }
      # Don't add a new section up to the end, because we will synchronize
      # the last breakpoint to the end of signal - else it would be done with:
@@ -590,17 +672,25 @@ proc NormalizeTrans {} {
 	  set t2 [$turn getAttr "startTime"]
 	  if {$t2 < $t1} {
 	    $turn setAttr "startTime" $t1
+	    append msg "shifted start time of overlapping turn $t2 -> $t1\n"
 	  } elseif {$t2 > $t1} {
 	    ::xml::element "Turn" [list "startTime" $t1 "endTime" $t2] \
 		-before $turn
+	    #append msg "inserted new turn between $t1-$t2\n"
 	  }
 	  set t1 [$turn getAttr "endTime"]
+	  # detect inconsistency in turns
+	  if {$t1 <= $t2} {
+	    append msg "WARNING - turn interval inconsistency $t2-$t1\n"
+	  }
 	}
 	set t2 [$sec getAttr "endTime"]
 	if {$t2 < $t1} {
 	  $turn setAttr "endTime" $t2
+	  append msg "shifted end time of overlapping turn $t1 -> $t2\n"
 	} elseif {$t2 > $t1} {
 	  ::xml::element "Turn" [list "startTime" $t1 "endTime" $t2] -in $sec
+	  #append msg "inserted new turn between $t1-$t2\n"
 	}
       }
 
@@ -612,6 +702,7 @@ proc NormalizeTrans {} {
 	    ::xml::element "Sync" [list "time" $t1] -begin $turn
 	 }
 	 if {$sync != "" && [$sync getAttr "time"] < $t1} {
+	   append msg "shifted synchro [$sync getAttr time] -> $t1\n"
 	   $sync setAttr "time" $t1
 	 }
 	 # Create data between non-contiguous breakpoints
@@ -633,6 +724,11 @@ proc NormalizeTrans {} {
 		      && [$data getAttr "nb"] == 1))} {
 		     ::xml::data "" -after $elem
 	       }
+	    } else {
+	      if {$t2 < $t1} {
+		# detect inconsistency in times
+		append msg "WARNING - element time inconsistency at $t2\n"
+	      }
 	    }
 	    set t1 $t2
 	 }
@@ -645,6 +741,7 @@ proc NormalizeTrans {} {
 	 }
       }
    }
+   return $msg
 }
 
 ################################################################
@@ -672,7 +769,7 @@ proc ConvertData {data} {
 	       }
 	    }
 	    set elem [::xml::element "Event" \
-			  [list "desc" $evt "extent" $extn] -after $data]
+		          [list "desc" $evt "extent" $extn] -after $data]
 	 }
 	 default {
 	    set elem [::xml::element "Comment" [list "desc" $evt] -after $data]
@@ -749,8 +846,8 @@ proc DisplayTrans {} {
 		     AddSegmt seg0 $t3 $t4 $txt $id
 		     # Test overlap with previous turn for display
 		     if {$time($tprev) > $time($t3)} {
-			ChangeSyncButton $idprev over1
-			ChangeSyncButton $id over2
+		        ChangeSyncButton $idprev over1
+		        ChangeSyncButton $id over2
 		     }
 		  }
 		  set t3 $t4
@@ -786,8 +883,20 @@ proc DisplayTrans {} {
 		   InsertScope $chn
 	       }
 	       "Event" - "Comment" {
-		  InsertEvent $chn
-		  append txt [StringOfEvent $chn]
+		   # Enable to display a " " between 2 contiguous events
+		   # In that case, if an event is deleted, it will be replaced by a " "                        
+		   set prevBrother     [$chn         getBrother * * -1]
+		   set prevPrevBrother [$prevBrother getBrother * * -1]
+		   if { ( [$prevBrother class] == "data" && [$prevBrother getData] == "" )
+		     && ([$prevPrevBrother class]=="element" && [$prevPrevBrother getType]=="Event")} {
+		            $prevBrother setData " "
+		            append txt " "
+		            InsertData $prevBrother
+		    }
+		   
+
+		    InsertEvent $chn
+		    append txt [StringOfEvent $chn]
 	       }
 	       "Who" {
 		  if {[$chn getAttr "nb"] > 1} {
@@ -796,6 +905,10 @@ proc DisplayTrans {} {
 		  }
 		  InsertWho $chn
 	       }
+	       default {
+		 InsertOther $chn
+		 append txt [StringOfOther $chn]
+	       }
 	       }
 	    }
 	    "data" {
@@ -881,6 +994,9 @@ proc TextFromSync {bp} {
 	       "Event" - "Comment" {
 		  append txt [StringOfEvent $tag]
 	       }
+	       default {
+		 append txt [StringOfOther $tag]
+	       }
 	    }
 	 }
       }
