@@ -1,4 +1,4 @@
-# RCS: @(#) $Id: Waveform.tcl,v 1.5 2003/07/16 16:46:56 barras Exp $
+# RCS: @(#) $Id: Waveform.tcl,v 1.10 2004/11/29 14:49:06 galliano Exp $
 
 # Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
@@ -6,93 +6,126 @@
 ################################################################
 
 proc CreateSoundFrame {f} {
-   global v
+    global v
 
-   frame $f -bd 1 -relief raised -bg $v(color,bg)
-   setdef v($f.w,height) 100
-   set wavfm [wavfm $f.w -padx 10 -bd 0 -bg $v(color,bg) \
+    # embedded frame for optional inclusion of video
+    frame $f; 
+    if {$v(view,$f)} { 
+	pack $f -fill both -side top
+    } 
+    set f $f.1
+    frame $f -bd 1 -relief raised -bg $v(color,bg)
+    setdef v($f.w,height) 100
+    set wavfm [wavfm $f.w -padx 10 -bd 0 -bg $v(color,bg) \
 		  -height $v($f.w,height) -selectbackground $v(color,bg-sel)]
-   axis $f.a -padx 10 -bd 0 -bg $v(color,bg) -font axis
-
-   set g $f.scr
-   frame $g -bg $v(color,bg)
-   set v($wavfm,scroll) [scrollbar $g.pos -orient horizontal -width 15\
-	-command [list ScrollTime $wavfm]]
-   pack $g.pos -fill x -side left -expand true -anchor n
-
-   # optional resolution scrollbar
-   frame $g.reso
-   label $g.reso.lab -text "Resolution" -font {fixed 10} -bd 0 -padx 10 -pady 0
-   set v($wavfm,scale) [scrollbar $g.reso.scrol -command [list ScrollReso $wavfm] -orient horizontal -width 8 -bd 1]
-   pack $g.reso.lab -side top
-   pack $g.reso.scrol -fill x -expand true
-   # Default : display resolution scrollbar
-   setdef v(view,$g.reso) 1
-   if {$v(view,$g.reso)} {
-      pack $g.reso -padx 0 -pady 0 -side right
-   }
-
-   pack $g -side top -fill x
-
-   pack $f.w -fill both -expand true -side top
-   pack $f.a -fill x -side top
-   pack $f -fill both -side top
-   set v($wavfm,sync) [list $wavfm $f.a]
-   lappend v(wavfm,list) $wavfm
-
-   # Register the bindings at the frame level
-   bindtags $wavfm [list $wavfm Wavfm $f . all]
-   bindtags $f.a [list $f.a Axis $f . all]
-
-   # Selection/cursor position with B1
-   bind $f <Button-1>  [list BeginCursorOrSelect $wavfm %X]
-   bind $f <B1-Motion> [list SelectMore $wavfm %X]
-   bind $f <ButtonRelease-1> [list EndCursorOrSelect $wavfm]
+    axis $f.a -padx 10 -bd 0 -bg $v(color,bg) -font axis
+    
+    set g $f.scr
+    frame $g -bg $v(color,bg)
+    set v($wavfm,scroll) [scrollbar $g.pos -orient horizontal -width 15\
+			      -command [list ScrollTime $wavfm]]
+    pack $g.pos -fill x -side left -expand true -anchor n
+    
+    # optional resolution scrollbar
+    frame $g.reso
+    label $g.reso.lab -text "Resolution" -font {fixed 10} -bd 0 -padx 10 -pady 0
+    set v($wavfm,scale) [scrollbar $g.reso.scrol -command [list ScrollReso $wavfm] -orient horizontal -width 8 -bd 1]
+    pack $g.reso.lab -side top
+    pack $g.reso.scrol -fill x -expand true
+    # Default : display resolution scrollbar
+    setdef v(view,$g.reso) 1
+    if {$v(view,$g.reso)} {
+	pack $g.reso -padx 0 -pady 0 -side right
+    }
+    
+    pack $g -side top -fill x
+    
+    pack $f.w -fill both -expand true -side top
+    pack $f.a -fill x -side top
+    pack $f -fill both -side top
+    set v($wavfm,sync) [list $wavfm $f.a]
+    lappend v(wavfm,list) $wavfm
+    
+    # Register the bindings at the frame level
+    bindtags $wavfm [list $wavfm Wavfm $f . all]
+    bindtags $f.a [list $f.a Axis $f . all]
+    
+    # Selection/cursor position with B1
+    bind $f <Button-1>  [list BeginCursorOrSelect $wavfm %X]
+    bind $f <B1-Motion> [list SelectMore $wavfm %X]
+    bind $f <ButtonRelease-1> [list EndCursorOrSelect $wavfm]
    bind $f <Shift-Button-1>  [list ExtendOldSelection $wavfm %X]
+    
+    # Extend selection with B2
+    bind $f <Button-2>  [list ExtendOldSelection $wavfm %X]
+    
+    # Contextual menus with B3
+    InitWavContextualMenu $f 
+    bind $f <Button-3>  [list tk_popup $v($wavfm,menu) %X %Y]
+    bind $f <Control-Button-1>  [list tk_popup $v($wavfm,menu) %X %Y]
+    
+    return $wavfm
+}
 
-   # Extend selection with B2
-   bind $f <Button-2>  [list ExtendOldSelection $wavfm %X]
+proc InitWavContextualMenu {f} {
 
-   # Contextual menus with B3
-   regsub -all {\.} $wavfm {_} name
-   set v($wavfm,menu) [add_menu .menu$name [subst {
-      {"Audio file"		cascade {
-	 {"Open audio file..." 	cmd {OpenAudioFile}}
-	 {"Add audio file..." 	cmd {OpenAudioFile add}}
-	 {""}
-      }}
-      {"Playback"		cascade {
-	 {"Play/Pause"		cmd {PlayOrPause}}
-	 {"Replay segment"	cmd {PlayCurrentSegmt}}
-	 {"Play around cursor"	cmd {PlayAround}}
-      }}
-      {"Position"		cascade {
-	 {"Forward"	cmd {PlayForward +1}}
-	 {"Backward"	cmd {PlayForward -1}}
-	 {"Previous"	cmd {MoveNextSegmt -1}}
-	 {"Next"	cmd {MoveNextSegmt +1}} 
-      }}
-      {"Resolution"		cascade {
-	 {"1 sec"	cmd {Resolution 1 $wavfm}}
-	 {"10 sec"	cmd {Resolution 10 $wavfm}}
-	 {"30 sec"	cmd {Resolution 30 $wavfm}}
-	 {"1 mn"	cmd {Resolution 60 $wavfm}}
-	 {"5 mn"	cmd {Resolution 300 $wavfm}}
-	 {""}
-	 {"up"		cmd {ZoomReso -1 $wavfm}}
-	 {"down"	cmd {ZoomReso +1 $wavfm}}
-	 {""}
-	 {"View all"	cmd {ViewAll $wavfm}}
-      }}
-      {"Display"		cascade {
-	 {"Resolution bar"	check v(view,$g.reso) -command {SwitchFrame $g.reso}}
-	 {"Reduce waveform"	cmd {WavfmHeight $wavfm [expr 1/1.2]}}
-	 {"Expand waveform"	cmd {WavfmHeight $wavfm 1.2}}
-	 {""}
-      }}
-   }]]
-   bind $f <Button-3>  [list tk_popup $v($wavfm,menu) %X %Y]
-   return $wavfm
+    # JOB: create the contextual menu on wave frame
+    #
+    # IN: f, the name of the wav frame, i.e. snd.1 or snd2.1
+    # OUT: nothing
+    # MODIFY: nothing
+    #
+    # Author: Claude Barras, Sylvain Galliano
+    # Version: 1.1
+    # Date: October 20, 2004
+
+    global v
+    
+    set wavfm $f.w
+    set g $f.src
+
+    regsub -all {\.} $wavfm {_} name
+    catch {destroy .menu$name}
+    set v($wavfm,menu) [add_menu .menu$name [subst {
+	{"Audio file"		cascade {
+	    {"Open audio file..." 	cmd {OpenAudioFile}}
+	    {"Add audio file..." 	cmd {OpenAudioFile add}}
+	    {"Save audio segment(s)"   cascade {
+		{"Selected..."     cmd   {SaveAudioSegment}}
+		{"Automatic..." cmd   {SaveAudioSegmentAuto}}
+	    }}
+	    {""}
+	}}
+	{"Playback"		cascade {
+	    {"Play/Pause"		cmd {PlayOrPause}}
+	    {"Replay segment"	cmd {PlayCurrentSegmt}}
+	    {"Play around cursor"	cmd {PlayAround}}
+	}}
+	{"Position"		cascade {
+	    {"Forward"	cmd {PlayForward +1}}
+	    {"Backward"	cmd {PlayForward -1}}
+	    {"Previous"	cmd {MoveNextSegmt -1}}
+	    {"Next"	cmd {MoveNextSegmt +1}} 
+	}}
+	{"Resolution"		cascade {
+	    {"1 sec"	cmd {Resolution 1 $wavfm}}
+	    {"10 sec"	cmd {Resolution 10 $wavfm}}
+	    {"30 sec"	cmd {Resolution 30 $wavfm}}
+	    {"1 mn"	cmd {Resolution 60 $wavfm}}
+	    {"5 mn"	cmd {Resolution 300 $wavfm}}
+	    {""}
+	    {"up"		cmd {ZoomReso -1 $wavfm}}
+	    {"down"	cmd {ZoomReso +1 $wavfm}}
+	    {""}
+	    {"View all"	cmd {ViewAll $wavfm}}
+	}}
+	{"Display"		cascade {
+	    {"Resolution bar"	check v(view,$g.reso) -command {SwitchFrame $g.reso}}
+	    {"Reduce waveform"	cmd {WavfmHeight $wavfm [expr 1/1.2]}}
+	    {"Expand waveform"	cmd {WavfmHeight $wavfm 1.2}}
+	    {""}
+	}}
+    }]]
 }
 
 proc SwitchSoundFrame {f} {
@@ -107,6 +140,7 @@ proc SwitchSoundFrame {f} {
      pack forget $f
    } else {
      pack $f -fill x
+
    }
 }
 
@@ -117,23 +151,27 @@ proc WavfmHeight {wavfm {val 1.0}} {
    $wavfm conf -height $v($wavfm,height)
 }
 
-proc ConfigWavfm {wavfm} {
+proc ConfigWavfm {wavfm {mode "reset"}} {
    global v
 
-   set v($wavfm,left)    0
-   set v($wavfm,size)    [setdef v($wavfm,resolution) 30]
-   if {$v(shape,cmd)=="" && $v($wavfm,size) > $v(shape,min)} {
+  if {$mode == "reset"} {
+    set v($wavfm,left)    0
+    set v($wavfm,size)    [setdef v($wavfm,resolution) 30]
+    if {$v(shape,cmd)=="" && $v($wavfm,size) > $v(shape,min)} {
       set v($wavfm,size) $v(shape,min)
-   }
-   $wavfm config -sound $v(sig,cmd) -shape $v(shape,cmd)
-   SynchroWidgets $wavfm
+    }
+  }
+  $wavfm config -sound $v(sig,cmd) -shape $v(shape,cmd)
+  if {$mode == "reset"} {
+    SynchroWidgets $wavfm
+  }
 }
 
-proc ConfigAllWavfm {} {
+proc ConfigAllWavfm {{mode "reset"}} {
    global v
 
    foreach wavfm $v(wavfm,list) {
-      ConfigWavfm $wavfm
+      ConfigWavfm $wavfm $mode
    }
 }
 
@@ -323,6 +361,7 @@ proc SetCursor {pos {hide 0}} {
       }
       DisplayMessage "Cursor : [Tim2Str [expr $v(curs,pos)+$v(sig,base)] 3]\t\t$v(sel,text)"
       SynchroToSignal $pos
+      ViewVideo $pos
    }
 }
 
@@ -584,75 +623,161 @@ proc UnZoom {{win ""}} {
       config_entry "Signal" "Unzoom selection" -state disabled
    }
 }
-# Save Audio Segment [as] 
-# Select the name, format and directory of the file to saved 
-# returns empty string if save failed (or was canceled)
-proc SaveAudioSegment {{as ""} {format ""}} {
-   global v 
 
-   set tmp "yes"
-   snack::sound player
- 
-   if {$format == ""} {
-       set format ".wav"
-   }
-   if {$v(sel,begin) == $v(sel,end) } {
-      set tmp [tk_messageBox -icon warning -message [Local "No audio segment selected !"] \
-                                -title "Warning" -type ok]
-      return ""
-   } else {
-       player conf -file $v(sig,name)
-       if { $v(sig,open) == "0" } {
-           set v(sig,name) "empty" 
-           set tmp [tk_messageBox -icon question -message [Local "   No audio file opened !\nThis will create an empty\n audio file ! Really save ?"] \
-                                               -title "Warning" -type yesno]
-       }
-       if {$tmp == "yes"} {
-             set types {
-		    { "Wave file" {.wav}}
-		    { "AU file" {.au}}
-		    { "Sound file" {.snd}}
-		    { "SD file" {.sd}}
-		    { "SMP file" {.smp}}
-		    { "AIFF" {.aiff}}
-		    { "RAW file" {.raw}}
-		    { "All Files" {*}}
-	       }
-	       set base [file root [file tail $v(sig,name)]]
-	       set zone [concat [format "%6.2f" $v(sel,begin)]-[format "%-6.2f" $v(sel,end)]]
-	       set name [tk_getSaveFile -filetypes $types -defaultextension $format \
-					-initialfile "$base\_$zone$format" -initialdir $v(trans,path) \
-					-title "Save audio segment"]
-	       if {$name != "" && [file extension $name] == ""} {
-		   append name $format
-	       }
-	       if {$name == ""} {return}
-	} else {return}
-     }
-   if [catch {
-	 PauseAudio
-	 set wavename $v(sig,name)
-	 if {$wavename == ""} return
+proc SaveAudioSegment {{auto ""}} {
 
-	 # if possible, keep previously open sound file
-	 if {[player cget -file] != $wavename} {
-	     player conf -file $wavename -channels $v(sig,channels) -frequency $v(sig,rate) -skiphead $v(sig,header) -guessproperties 1
-	 }
-	 if { $v(sig,cmd) != "" } { 
-	     set rate [$v(sig,cmd) cget -frequency]
-	     player write $name -start [expr int($v(sel,begin)*$rate)] -end [expr int($v(sel,end)*$rate)]
-	 } else {
+    # JOB: save an audio selection
+    #
+    # IN: nothing
+    # OUT: nothing
+    # MODIFY: nothing
+    #
+    # Author: Sylvain Galliano
+    # Version: 1.2
+    # Date: November 26, 2004
+    # 
+    # Save Audio Segment in normal or automatic mode 
+    # Select the name, format and directory of the file to saved 
+    # returns empty string if save failed (or was canceled)
+
+    global v 
+    
+    snack::sound player
+    
+    if { $auto == "" && $v(sel,begin) == $v(sel,end) } {
+	tk_messageBox -icon warning -message [Local "No audio segment selected !"] -title "Warning" -type ok
+	return ""
+    } else {
+	if { $v(sig,cmd) == "" } {
+	    set v(sig,name) "empty" 
+	    set rep [tk_messageBox -icon question -message [Local "   No audio file opened !\nThis will create an empty\n audio file ! Really save ?"] \
+			 -title "Warning" -type yesno]
+	    if {$rep == "no"} {
+		return
+	    } else {
+		set format ".wav"
+	    }
+	} else {
+	    set format [file extension $v(sig,name)]
+	}
+	set types {
+	    { "Wave file" {.wav}}
+	    { "AU file" {.au}}
+	    { "Sound file" {.snd}}
+	    { "SD file" {.sd}}
+	    { "SMP file" {.smp}}
+	    { "AIFF" {.aiff}}
+	    { "RAW file" {.raw}}
+	    { "All Files" {*}}
+	}
+	set base [file root [file tail $v(sig,name)]]
+	if {$auto == ""} {
+	    set name [tk_getSaveFile -filetypes $types -initialfile "$base\_$zone$format" -initialdir $v(trans,path) -title "Save audio segment"]
+	    if {$name == ""} return
+	}
+	if [catch {
+	    player conf -file $v(sig,name)
+	    PauseAudio
+	    
+	    # if possible, keep previously open sound file
+	    player conf -file $v(sig,name) -channels $v(sig,channels) -frequency $v(sig,rate) -skiphead $v(sig,header) -guessproperties 1
+	    if { $v(sig,cmd) != "" } { 
+		set rate [$v(sig,cmd) cget -frequency]
+		if {$auto == "" } {
+		    set zone [concat [format "%6.2f" $v(sel,begin)]-[format "%-6.2f" $v(sel,end)]]
+		    player write $name -start [expr int($v(sel,begin)*$rate)] -end [expr int($v(sel,end)*$rate)]
+		} else {
+		    #Automatic mode
+		    set loop ""
+		    foreach segment {"Section" "Turn" "Sync"} {
+			if {$v($segment,loop)} {
+			    lappend loop $segment
+			}
+		    }
+		    if {$loop != ""} {
+			set tot 0
+			foreach segment $loop {
+			    set cpt 0
+			    set begin $v(sig,min)
+			    set end 0
+			    set max $v(sig,max)
+			    SetCursor $begin
+			    while {$begin < $max} {
+				set nb $v(segmt,curr)
+				set tag [GetSegmtId $nb]
+				if {$segment == "Section"} {
+				    set sec [[$tag getFather] getFather]
+				    set id [::section::long_name $sec]
+				}
+				if {$segment == "Turn"} {
+				    set tur [$tag getFather]
+				    set spk [$tur getAttr "speaker"]
+				    set spk [::speaker::name $spk]
+				    set id [string trim $spk "_"]
+				}
+				set alnum {[^[:alnum:]]+}
+				regsub -all $alnum $id "_" id
+				
+				TextNext$segment +1
+				set end [GetCursor]
+				if {$end == $begin || $end == 0} {
+				    set end $max
+				}
+				set zone [concat [format "%6.2f" $begin]-[format "%-6.2f" $end]]
+				set num [format "%03.0f" [incr cpt]]
+				set name [file join $v(saveaudioseg,dir) "$base\_$segment$num\_$id\_$zone$format"]
+				regsub -all "__" $name "_" name
+				player write $name -start [expr int($begin*$rate)] -end [expr int($end*$rate)]
+				set tot [incr tot]
+				set begin $end
+			    }
+			    set v($segment,loop) 0
+			}
+		    } 
+		}
+	    } else {
 		set time [expr int([format "%6.3f" [expr $v(sel,end) - $v(sel,begin)]]*16000)]
 		set f [snack::filter generator 0.0 0 0.0 sine $time] 
 		set s [snack::sound]
 		$s filter $f
 		$s write $name   
-	   }  
-   } res] {
-	 tk_messageBox -message [format [Local "%s not saved !!"] $name] -type ok -icon error
-	 return "" 
-      } else {
-              tk_messageBox -message [format [Local "%s saved !!"] $name] -type ok -icon info
-        }
-   return $name
+	    }  
+	} res] {
+	    tk_messageBox -message "[Local "Error, wave segment(s) not saved !!"] $res" -type ok -icon error
+	    return "" 
+	} else {
+	    if {$loop != "" } {
+		tk_messageBox -message [format [Local "Ok, %s wave segment(s) saved !!"] $tot] -type ok -icon info
+	    } else return
+	}
+    }
+}
+
+proc SaveAudioSegmentAuto {} {
+    # JOB: open a dialog box to define the option for saving automaticaly each kind of wave segment (turn, section, sync).
+    #      you have to choose the destination directory and the elemnt to save  
+    #
+    # IN: nothing
+    # OUT: nothing
+    # MODIFY: nothing
+    #
+    # Author: Sylvain Galliano
+    # Version: 1.0
+    # Date: November 29, 2004
+
+    global v
+
+    set w [CreateModal .save [Local "Save audio segments options"]]
+    set f [frame $w.top -relief raised -bd 1]
+    pack $f -side top -fill both
+    set i 0
+    foreach segment {"Section" "Turn" "Sync"} {
+	set b [checkbutton $f.rad[incr i] -var v($segment,loop) -text [Local $segment]]
+	grid $b -row 0 -column "$i"  -sticky w -padx 3m -pady 3m
+    }
+    EntryFrame $w.dir [Local "Destination directory"] v(saveaudioseg,dir) 50
+    set v(saveaudioseg,dir) [pwd]
+    if {[OkCancelModal $w $w {"OK" "Cancel"}] == "OK"} {
+	SaveAudioSegment auto
+    } else return
 }
