@@ -2,7 +2,9 @@
 #\
 exec wish "$0" "$@"
 
-# Copyright (C) 1998, DGA - part of the Transcriber program
+# RCS: @(#) $Id: XmlDtd.tcl,v 1.2 2002/07/10 07:36:37 barras Exp $
+
+# Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
 # WWW:          http://www.etca.fr/CTA/gip/Projets/Transcriber/Index.html
 # Author:       Claude Barras
@@ -46,7 +48,7 @@ namespace eval ::xml::dtd {
    }
 
    # read new external DTD and switch to dynamic validating mode
-   proc read {fileName args} {
+   proc xml_read {fileName args} {
       init
       eval ::xml::parser::read_file [list $fileName] $args -type dtd
       active 1
@@ -459,7 +461,7 @@ namespace eval ::xml::dtd {
 	    if {$decimal != ""} {
 	       set char [format {%c} $decimal]
 	       # Test if $char is in the range of valid chars
-	       if {![regexp $Char $char]} {
+	       if {$decimal >= 65536 || ![regexp "\[$Char]" $char]} {
 		  error "forbidden character entity &\#x[format %x $decimal];"
 	       }
 	       append done $char
